@@ -1,10 +1,10 @@
-/* $Id: board.cc,v 1.59 2001/10/14 17:36:26 bergo Exp $ */
+/* $Id: board.cc,v 1.62 2002/02/13 03:53:56 bergo Exp $ */
 
 /*
 
     eboard - chess client
     http://eboard.sourceforge.net
-    Copyright (C) 2000-2001 Felipe Paulo Guazzi Bergo
+    Copyright (C) 2000-2002 Felipe Paulo Guazzi Bergo
     bergo@seul.org
 
     This program is free software; you can redistribute it and/or modify
@@ -163,6 +163,7 @@ Board::Board(bool withbugpane=false) : WidgetProxy() {
   FreeMove=false;
   dr_active=false;
   FlipInvert=false;
+  EditMode=false;
 
   info[0][0]=info[1][0]=info[2][0]=info[3][0]=info[4][0]=info[5][0]=0;
 
@@ -171,9 +172,9 @@ Board::Board(bool withbugpane=false) : WidgetProxy() {
   repaint_due=0;
   frozen_lvl=0;
 
-  f1=loadFallbackFont(1);
-  f2=loadFallbackFont(2);
-  f3=loadFallbackFont(3);
+  f1=global.loadFont(EF_PlayerFont);
+  f2=global.loadFont(EF_ClockFont);
+  f3=global.loadFont(EF_InfoFont);
 
   if (global.ShowCoordinates) { borx=20; bory=20; } else { borx=bory=0; }
   morey=0;
@@ -194,7 +195,9 @@ Board::Board(bool withbugpane=false) : WidgetProxy() {
   bugpane=gtk_hbox_new(FALSE,0);
 
   yidget=gtk_drawing_area_new();
-  gtk_widget_set_events(yidget,GDK_EXPOSURE_MASK|GDK_BUTTON_PRESS_MASK|GDK_BUTTON_RELEASE_MASK|GDK_BUTTON1_MOTION_MASK|GDK_POINTER_MOTION_HINT_MASK);
+  gtk_widget_set_events(yidget,GDK_EXPOSURE_MASK|GDK_BUTTON_PRESS_MASK|
+			GDK_BUTTON_RELEASE_MASK|GDK_BUTTON1_MOTION_MASK|
+			GDK_POINTER_MOTION_HINT_MASK);
   buffer=gdk_pixmap_new(MainWindow::RefWindow,SCRAP_WIDTH,SCRAP_WIDTH,-1);
   clkbar=gdk_pixmap_new(MainWindow::RefWindow,250,SCRAP_WIDTH,-1);
   scrap=(rgbptr)g_malloc(SCRAP_WIDTH*SCRAP_WIDTH*3);
@@ -228,37 +231,6 @@ Board::Board(bool withbugpane=false) : WidgetProxy() {
   gtk_widget_show(widget);
 }
 
-GdkFont * Board::loadFallbackFont(int i) {
-  GdkFont *f;
-  switch(i) {
-
-  case 1: // player font (default 18 pt)
-    f=gdk_font_load(global.PlayerFont);
-    if (!f) f=gdk_font_load("-*-helvetica-bold-r-normal--17-*-*-*-*-*-*");
-    if (!f) f=gdk_font_load("-*-helvetica-bold-r-normal--14-*-*-*-*-*-*");
-    if (!f) f=gdk_font_load("helvetica");
-    break;       
-
-  case 2: // clock font (default 34 pt)
-    f=gdk_font_load(global.ClockFont);
-    if (!f) f=gdk_font_load("-*-helvetica-bold-r-normal--34-*-*-*-*-*-*");
-    if (!f) f=gdk_font_load("-*-helvetica-bold-r-normal--25-*-*-*-*-*-*");
-    if (!f) f=gdk_font_load("-*-helvetica-bold-r-normal--24-*-*-*-*-*-*");
-    if (!f) f=gdk_font_load("helvetica");
-    break;
-
-  case 3: // info font (default 12 pt)
-    f=gdk_font_load(global.InfoFont);
-    if (!f) f=gdk_font_load("-*-helvetica-medium-r-normal--11-*-*-*-*-*-*");
-    if (!f) f=gdk_font_load("-*-helvetica-medium-r-normal--10-*-*-*-*-*-*");
-    if (!f) f=gdk_font_load("helvetica");
-    break;
-  default:
-    return NULL;
-  }
-  return f;
-}
-
 bool Board::hasGame() {
   return(mygame!=0);
 }
@@ -338,9 +310,9 @@ void Board::reloadFonts() {
   if (f2) gdk_font_unref(f2);
   if (f3) gdk_font_unref(f3);
 
-  f1=loadFallbackFont(1);
-  f2=loadFallbackFont(2);
-  f3=loadFallbackFont(3);
+  f1=global.loadFont(EF_PlayerFont);
+  f2=global.loadFont(EF_ClockFont);
+  f3=global.loadFont(EF_InfoFont);
 
   if ((!f1)||(!f2)||(!f3)) {
     cerr << "<Board::Board> ** failed to load one or more board fonts\n";
@@ -466,6 +438,10 @@ void Board::refreshPieceSet() {
   queueRepaint();
 }
 
+ChessGame * Board::getGame() {
+  return(mygame);
+}
+
 void Board::setGame(ChessGame *game) {
   global.debug("Board","setGame");
   mygame=game;
@@ -571,8 +547,7 @@ void Board::repaint() {
 
 void Board::renderClock() {
   GdkGC *gc;
-  int i,j,k,sq,bw,bh,ch,th;
-  GdkFont *fa,*fb;
+  int i,j,k,sq,bh,ch,th;
   char z[64];
   bool ef;
 
@@ -582,75 +557,92 @@ void Board::renderClock() {
   sq=sqside;
 
   gc=gdk_gc_new(clkbar);
-  gdk_rgb_gc_set_foreground(gc,0);
-  gdk_draw_rectangle(clkbar,gc,TRUE,0,0,250,SCRAP_WIDTH);
 
-  bw=170; bh=56; ch=20; th=36;
-  fa=f1; fb=f2;
+  C.prepare(clkbar,gc,0,0,250,SCRAP_WIDTH);
+  C.setColor(0);
+  C.drawRect(0,0,C.W,C.H,true);
+  C.H = Height();
+
+  bh=56; ch=20; th=36; // font heights
+  C.setFont(0,f1);
+  C.setFont(1,f2);
+  C.setFont(2,f3);
 
   if (sq*3 < bh) {
     bh=28; ch=14; th=14;
-    fa=fb=f3;
+    C.setFont(0,f3);
+    C.setFont(1,f3);
+    C.setFont(2,f3);
   }
 
+  // draw the clock time and White/Black labels
+
   ef=effectiveFlip();
-  if (ef) i=Height()-(th+10); else i=ch+10;
+  if (ef) i= C.H - (th+10); else i = ch+10;
   j=16;
-  
-  gdk_rgb_gc_set_foreground(gc,0xffffff);
+
+  C.setColor(0xffffff);
   if (clock.getActive()>0) {
-    gdk_draw_rectangle(clkbar,gc,TRUE,15,i-(ch+2),200-30,bh+4);
-    gdk_rgb_gc_set_foreground(gc,0);
+    C.drawRect(15, i - (ch+2), 200-30, bh+4, true);
+    C.setColor(0);
   }
 
-  gdk_draw_string(clkbar,fa,gc,j,i,"Black");
+  C.drawString(j,i,0,"Black");
   clockString(clock.getValue(1),z);
-  gdk_draw_string(clkbar,fb,gc,j,i+th,z);
+  C.drawString(j,i+th,1,z);
 
-  if (ef) i=ch+10; else i=Height()-(th+10);
+  if (ef) i=ch+10; else i= C.H - (th+10);
 
-  gdk_rgb_gc_set_foreground(gc,0xffffff);
+  C.setColor(0xffffff);
   if (clock.getActive()<0) {
-    gdk_draw_rectangle(clkbar,gc,TRUE,15,i-(ch+2),200-30,bh+4);
-    gdk_rgb_gc_set_foreground(gc,0);
+    C.drawRect(15, i - (ch+2), 200-30, bh+4, true);
+    C.setColor(0);
   }
-  gdk_draw_string(clkbar,fa,gc,j,i,"White");
+  C.drawString(j,i,0,"White");
   clockString(clock.getValue(0),z);
-  gdk_draw_string(clkbar,fb,gc,j,i+th,z);
+  C.drawString(j,i+th,1,z);
+
+  C.consumeTop(bh+20);
+  C.consumeBottom(bh+20);
 
   // if we have enough space, render player names too
-  if (((Height() - bh*2) > 60)&&(mygame)) {
-    i=ch+10+th+ch+10;
-    gdk_rgb_gc_set_foreground(gc,0xffffff);
-    gdk_draw_string(clkbar,fa,gc,j,i,mygame->getPlayerString(ef?0:1));
-    i=Height()-(th+10)-ch-10;
-    gdk_draw_string(clkbar,fa,gc,j,i,mygame->getPlayerString(ef?1:0));
+  if ( (mygame) && (C.H > 60) ) {
+    C.setColor(0xffffff);
+    C.drawString(j,C.Y + ch,0, mygame->getPlayerString(ef?0:1) );
+    C.drawString(j,C.Y + C.H,0, mygame->getPlayerString(ef?1:0) );
+    C.consumeTop(ch+8);
+    C.consumeBottom(ch+8);
   }
-  
+
   cleanTargets();
 
+  // now render the information lines
+
   if (mygame) {
-    int lr;
-    lr=Height()-bh*2-ch*2;
-    i=ch+10+th+ch+10+16+12;
-    gdk_rgb_gc_set_foreground(gc,0xffffff);
-    for(k=0;k<5;k++,i+=16,lr-=16) {
-      if (lr < 16) break;
-      gdk_draw_string(clkbar,f3,gc,j,i,info[k]);
+    C.setColor(0xffffff);
+
+    for(k=0;k<5;k++) {
+      if (C.H < 16) break;
+      C.drawString(j, C.Y + 16, 2, info[k]);
+      C.consumeTop(16);
     }
     
     // house string is info[5]
 
     if (strlen(info[5])) {      
-      if ((!global.DrawHouseStock)||(lr<96)) {
-	if (lr>=16)
-	  gdk_draw_string(clkbar,f3,gc,j,i,info[5]);
+      if ((!global.DrawHouseStock)||(C.H < 96)) {
+	if (C.H >= 16) {
+	  C.drawString(j,C.Y+16,2,info[5]);
+	  C.consumeTop(16);
+	}
       } else {
 	// draw pieces in stock
 	piece pk[5]={PAWN,ROOK,KNIGHT,BISHOP,QUEEN};
 	int st[5][2];
 	int si,sj,we,be;
 	char *p;
+
+	// build stock info from string
 	for(si=0;si<5;si++) { st[si][0]=st[si][1]=0; }
 	sj=0;
 	for(p=info[5];*p!=0;p++) {
@@ -663,43 +655,106 @@ void Board::renderClock() {
 	  case ']': sj=1; break;
 	  }
 	}
+
+	// draw stock pieces
 	if (ef) { we=0; be=48; } else { we=48; be=0; }
 	for(si=0;si<5;si++) {
 	  if (st[si][0]) {
-	    Board::vpieces.drawPiece(clkbar,gc,40,40*si,i+we,pk[si]|WHITE);
-	    gdk_rgb_gc_set_foreground(gc,0xffffff);
+	    Board::vpieces.drawPiece(clkbar,gc,40,40*si,C.Y+we,pk[si]|WHITE);
 	    sprintf(z,"%d",st[si][0]);
-	    gdk_draw_string(clkbar,f3,gc,40*si+20,i+48+we,z);
+	    C.setColor(0xffffff);
+	    C.drawString(40*si+20, C.Y+48+we, 2, z);
 	    if (mygame->MyColor==WHITE)
-	      addTarget(new DropSource(pk[si]|WHITE,Width()+40*si,i+we,40,48));
+	      addTarget(new DropSource(pk[si]|WHITE,Width()+40*si,C.Y+we,40,48));
 	  }
 	  if (st[si][1]) {
-	    Board::vpieces.drawPiece(clkbar,gc,40,40*si,i+be,pk[si]|BLACK);
-	    gdk_rgb_gc_set_foreground(gc,0xffffff);
+	    Board::vpieces.drawPiece(clkbar,gc,40,40*si,C.Y+be,pk[si]|BLACK);
+
 	    sprintf(z,"%d",st[si][1]);
-	    gdk_draw_string(clkbar,f3,gc,40*si+20,i+48+be,z);
+	    C.setColor(0xffffff);
+	    C.drawString(40*si+20,C.Y+48+be, 2, z);
 	    if (mygame->MyColor==BLACK)
-	      addTarget(new DropSource(pk[si]|BLACK,Width()+40*si,i+be,40,48));
+	      addTarget(new DropSource(pk[si]|BLACK,Width()+40*si,C.Y+be,40,48));
 	  }
-	}
+	} // for
+
+	C.consumeTop(100);
+      } // text | graphic stock
+    } // if strlen info[5]
+
+    // scratch-board pieces and buttons
+    if (EditMode) {
+      C.consumeTop(10);
+      i = 32;
+
+      C.setColor(0x404040);
+      C.drawRect(10,C.Y,i*6,i*2+8,true);
+      C.setColor(0xffffff);
+      C.drawRect(10,C.Y,i*6,i*2+8,false);
+
+      for(k=0;k<6;k++) {
+	Board::vpieces.drawPiece(clkbar,gc, i, 10+i*k, C.Y,WHITE|(k+1));
+	Board::vpieces.drawPiece(clkbar,gc, i, 10+i*k, C.Y+i+8,BLACK|(k+1));
+	addTarget(new DropSource(WHITE|(k+1),Width()+10+i*k,C.Y,i,i));
+	addTarget(new DropSource(BLACK|(k+1),Width()+10+i*k,C.Y+i+8,i,i));
       }
-      i+=100;
+
+      C.consumeTop(i*2+16);
+
+      C.setColor(0x404040);
+      j=C.stringWidth(2,"Empty");
+      C.drawRect(10,C.Y,j+10,20,true);
+      C.setColor(0xffffff);
+      C.drawRect(10,C.Y,j+10,20,false);
+      C.drawString(15,C.Y+14,2,"Empty");
+
+      addTarget(new DropSource(EMPTY,Width()+10,C.Y,j+10,20,false));
+
+      k=j;
+      C.setColor(0x404040);
+      j=C.stringWidth(2,"Start Position");
+      C.drawRect(40+k,C.Y,j+10,20,true);
+      C.setColor(0xffffff);
+      C.drawRect(40+k,C.Y,j+10,20,false);
+      C.drawString(45+k,C.Y+14,2,"Start Position");
+
+      addTarget(new DropSource(WHITE|BLACK,Width()+40+k,C.Y,j+10,20,false));
+
+      C.consumeTop(20);
     }
+
     // draw mouse if protocol menu available
     if (global.protocol)
       if (global.protocol->getPlayerActions()) {
-	gdk_rgb_gc_set_foreground(gc,0xffffff);
-	gdk_draw_rectangle(clkbar,gc,TRUE,170,i+8,16,18);
-	gdk_draw_line(clkbar,gc,177,i+8,177,i+4);
-	gdk_rgb_gc_set_foreground(gc,0xffcc00);
-	gdk_draw_rectangle(clkbar,gc,TRUE,180,i+8,6,8);
-	gdk_rgb_gc_set_foreground(gc,0);
-	gdk_draw_line(clkbar,gc,170,i+16,186,i+16);
-	gdk_draw_line(clkbar,gc,175,i+8,175,i+16);
-	gdk_draw_line(clkbar,gc,180,i+8,180,i+16);
+	C.setColor(0xffffff);
+	C.drawRect(170,C.Y+8,16,18,true);
+	C.drawLine(177,C.Y+8,177,C.Y+4);
+	C.setColor(0xffcc00);
+	C.drawRect(180,C.Y+8,6,8,true);
+	C.setColor(0);
+	C.drawLine(170,C.Y+16,186,C.Y+16);
+	C.drawLine(175,C.Y+8,175,C.Y+16);
+	C.drawLine(180,C.Y+8,180,C.Y+16);
+	C.consumeTop(20);
       }	
   }
 
+  C.consumeTop(16);
+  j=16;
+
+  if (C.H > 16) {
+    if (position.getAnnotation()) {
+      C.setColor(0x252535);
+      C.drawRect(j-5,C.Y,170,C.H,true);
+      C.setColor(0x4545ff);
+      C.drawRect(j-5,C.Y,170,C.H,false);
+
+      C.setColor(0xffffff);
+      C.drawBoxedString(j,C.Y,160,C.H,2, position.getAnnotation(), " ");
+    }
+  }
+
+  gdk_gc_destroy(gc);
   clock_ch=0;
 }
 
@@ -867,8 +922,9 @@ void Board::setCanMove(int value) {
     } else {
       if (position.isDropLegal(pmp,pmx[1],pmy[1],
 			       mygame->MyColor,mygame->Variant)) {
-	position.moveDrop(pmp|mygame->MyColor, pmx[1], pmy[1]);
-	mygame->sendDrop(pmp|mygame->MyColor, pmx[1], pmy[1]);
+	if (pmp&COLOR_MASK == 0) pmp|=mygame->MyColor;
+	position.moveDrop(pmp, pmx[1], pmy[1]);
+	mygame->sendDrop(pmp, pmx[1], pmy[1]);
       }
     }
     premoveq=0;
@@ -977,6 +1033,7 @@ void Board::drawBall(GdkGC *gc, int x,int y,int color,int radius) {
 void Board::drop(piece p) {
   global.debug("Board","drop");
   if (!mygame) return;
+  if (p&COLOR_MASK == 0) p|=mygame->MyColor;
   if ((!allowMove)&&(!FreeMove)) {
     if (global.Premove) {
       premoveq=1;
@@ -988,7 +1045,7 @@ void Board::drop(piece p) {
       return;
   } else {
     if ((global.CheckLegality)&&(!FreeMove))
-      if (!position.isDropLegal(p|mygame->MyColor, dropsq[0], dropsq[1],
+      if (!position.isDropLegal(p&PIECE_MASK, dropsq[0], dropsq[1],
 				mygame->MyColor,mygame->Variant))
 	{
 	  char z[128];
@@ -997,8 +1054,8 @@ void Board::drop(piece p) {
 	  global.status->setText(z);
 	  goto drop_cleanup;
 	}
-    position.moveDrop(p|mygame->MyColor, dropsq[0], dropsq[1]);
-    mygame->sendDrop(p|mygame->MyColor, dropsq[0], dropsq[1]);
+    position.moveDrop(p, dropsq[0], dropsq[1]);
+    mygame->sendDrop(p, dropsq[0], dropsq[1]);
   }
 
  drop_cleanup:
@@ -1996,20 +2053,30 @@ gboolean board_button_press_event(GtkWidget *widget,GdkEventButton *be,
   if ((x>7)||(y>7)) {
     DropSource *ds;
     if (be->button == 1) {
-      ds=me->hitTarget((int)(be->x),(int)(be->y));
+      ds=me->hitTarget((int)(be->x),(int)(be->y));      
       if (ds) {
-	me->dr_active=true;
-	me->dr_fto=false;
-	me->dr_fromstock=true;
-	me->dr_step=0;
-	me->dr_p=ds->P;
-	me->dr_ox = me->sqside / 2;
-	me->dr_oy = me->sqside / 2;
-	me->dr_c=0; // non-sense
-	me->dr_r=0;
-	gtk_timeout_add(90,board_drag_start_timeout,
-			(gpointer)(&(me->dr_fto)));
-      }      
+	if (ds->dragged) {
+	  me->dr_active=true;
+	  me->dr_fto=false;
+	  me->dr_fromstock=true;
+	  me->dr_step=0;
+	  me->dr_p=ds->P;
+	  me->dr_ox = me->sqside / 2;
+	  me->dr_oy = me->sqside / 2;
+	  me->dr_c=0; // non-sense
+	  me->dr_r=0;
+	  gtk_timeout_add(90,board_drag_start_timeout,
+			  (gpointer)(&(me->dr_fto)));
+	} else {
+	  // empty / start pos buttons
+	  if ((me->EditMode)&&(me->mygame)) {
+	    switch(ds->P) {
+	    case EMPTY: me->mygame->editEmpty(); break;
+	    case WHITE|BLACK: me->mygame->editStartPos(); break;
+	    }
+	  }
+	}
+      }
     }
     if (be->button == 3) {
       me->popupProtocolMenu( (int) be->x, (int) be->y, be->time );
@@ -2233,10 +2300,11 @@ gboolean board_animate(gpointer data) {
 
 // ------------- targets
 
-DropSource::DropSource(piece p, int x1,int y1,int w,int h) {
+DropSource::DropSource(piece p, int x1,int y1,int w,int h,bool d=true) {
   X=x1; Y=y1;
   X2=X+w; Y2=Y+h;
   P=p;
+  dragged = d;
 }
 
 bool DropSource::hit(int x,int y) {
@@ -2290,3 +2358,9 @@ void board_ptell(GtkWidget *b,gpointer data) {
     global.network->writeLine(z);
   }
 }
+
+EditBoard::EditBoard(ChessGame *cg) : Board(false) {
+  EditMode = true;
+  FreeMove = true;
+  setGame(cg);
+}
