@@ -1,10 +1,10 @@
-/* $Id: chess.cc,v 1.42 2001/10/24 11:10:21 bergo Exp $ */
+/* $Id: chess.cc,v 1.49 2002/02/13 03:53:56 bergo Exp $ */
 
 /*
 
     eboard - chess client
     http://eboard.sourceforge.net
-    Copyright (C) 2000-2001 Felipe Paulo Guazzi Bergo
+    Copyright (C) 2000-2002 Felipe Paulo Guazzi Bergo
     bergo@seul.org
 
     This program is free software; you can redistribute it and/or modify
@@ -31,6 +31,7 @@
 #include <unistd.h>
 #include <ctype.h>
 #include <time.h>
+#include <errno.h>
 #include "chess.h"
 #include "movelist.h"
 #include "tstring.h"
@@ -50,8 +51,7 @@ ChessGame::ChessGame() {
   Variant=REGULAR;
   memset(PlayerName[0],0,64);
   memset(PlayerName[1],0,64);
-  Rating[0][0]=0;
-  Rating[1][0]=0;
+  Rating[0][0]=Rating[1][0]=0;
   last_half_move=-1;
   myboard=0;
   clock_regressive=1;
@@ -61,6 +61,10 @@ ChessGame::ChessGame() {
   over=0;
   MyColor=WHITE;
   StopClock=0;
+  LocalEdit=false;
+  Loaded=true;
+  PGNSource[0]=0;
+  SourceOffset=0;
 }
 
 ChessGame::ChessGame(int _number,int _tyme,int _inc, int _rated,
@@ -81,6 +85,49 @@ ChessGame::ChessGame(int _number,int _tyme,int _inc, int _rated,
   over=0;
   MyColor=WHITE;
   StopClock=0;
+  Rating[0][0]=Rating[1][0]=0;
+  LocalEdit=false;
+  Loaded=true;
+  PGNSource[0]=0;
+  SourceOffset=0;
+}
+
+ChessGame::ChessGame(ChessGame *src) {
+  list<Position *>::iterator i;
+  Position *p;
+  bool cset=false;
+
+  GameNumber = src->GameNumber;
+  STime = src->STime;
+  Inc = src->Inc;
+  Rated = src->Rated;
+  Variant = src->Variant;
+  strcpy(PlayerName[0],src->PlayerName[0]);
+  strcpy(PlayerName[1],src->PlayerName[1]);
+  last_half_move = src->last_half_move;
+  myboard=0;
+  mymovelist=0;
+  over=0;
+  MyColor=WHITE;
+  StopClock=0;
+  Rating[0][0]=Rating[1][0]=0;
+  LocalEdit=false;
+
+  Loaded=true;
+  PGNSource[0]=0;
+  SourceOffset=0;
+
+  // clone move list
+  for(i=src->moves.begin();i!=src->moves.end();i++) {
+    p=new Position();
+    (*p) = (*(*i));
+    moves.push_back(p);
+    if (src->cursor == i) { cursor = moves.end(); cursor--; cset = true; }
+  }
+
+  if (moves.empty())
+    moves.push_back(new Position());
+  if (!cset) { cursor = moves.end();  cursor--; }
 }
 
 ChessGame::~ChessGame() {
@@ -174,8 +221,28 @@ void ChessGame::retreat(int nmoves) {
   cursor--;
 }
 
+void ChessGame::fixExamineZigZag(Position &suspect) {
+  list<Position *>::iterator i;
+
+  for(i=moves.begin();i!=moves.end();i++)
+    if ( (*(*i)) == suspect ) 
+      if (!strcmp(suspect.getLastMove(), (*i)->getLastMove() )) {
+	junkMoves(i,moves.end());
+	return;
+      }
+}
+
+void ChessGame::junkMoves(list<Position *>::iterator from,
+			  list<Position *>::iterator to) {
+  list<Position *>::iterator i;
+  for(i=from;i!=to;i++)
+    delete(*i);
+  moves.erase(from,to);
+}
+
 void ChessGame::updatePosition(Position &p,int movenum,int blacktomove,
-			       int wclock,int bclock,char *infoline, bool sndflag=false) {
+			       int wclock,int bclock,char *infoline, 
+			       bool sndflag=false) {
   Position *np;
   char z[32];
 
@@ -195,6 +262,9 @@ void ChessGame::updatePosition(Position &p,int movenum,int blacktomove,
   *np=p;
   np->setLastMove(infoline);
   //  cerr << "LastMove = [" << infoline << "]\n";
+
+  fixExamineZigZag(*np);
+
   moves.push_back(np);
   cursor=moves.end();
   cursor--;
@@ -317,13 +387,67 @@ void ChessGame::moveListClosed() {
   mymovelist=0;
 }
 
+void ChessGame::editEmpty() {
+  Position *p;
+  int i,j;
+
+  if (!LocalEdit) return;
+
+  moves.clear();
+  last_half_move=1;
+
+  p = new Position();
+  for(i=0;i<8;i++) for(j=0;j<8;j++) p->setPiece(i,j,EMPTY);
+  moves.push_back(p);
+  cursor=moves.end();
+  cursor--;
+  if (myboard) { myboard->setInfo(2,(*cursor)->getLastMove()); myboard->update(); }
+}
+
+void ChessGame::editStartPos() {
+  Position *p;
+
+  if (!LocalEdit) return;
+
+  moves.clear();
+  last_half_move=1;
+
+  p = new Position();
+  moves.push_back(p);
+  cursor=moves.end();
+  cursor--;
+  if (myboard) { myboard->setInfo(2,(*cursor)->getLastMove()); myboard->update(); }
+}
+
 void ChessGame::sendDrop(piece p, int x, int y) {
-  global.protocol->sendDrop(p,x,y);
+  Position *o;
+  list<Position *>::iterator z;
+
+  if (LocalEdit) {
+    o = new Position();
+    (*o) = *(*cursor);
+    o->setPiece(x,y,p);
+
+    z=cursor; z++; 
+    junkMoves(z,moves.end());
+
+    moves.push_back(o);
+    last_half_move++;
+    cursor=moves.end();
+    cursor--;
+    if (myboard) { myboard->setInfo(2,(*cursor)->getLastMove()); myboard->update(); }
+  } else {
+    if (global.protocol) global.protocol->sendDrop(p&PIECE_MASK,x,y);
+  }
+
 }
 
 void ChessGame::sendMove(int x1,int y1,int x2,int y2) {
   Position *p;
   int promote=0;
+  piece q;
+  list<Position *>::iterator z;
+  char v[16];
 
   if (!moves.empty()) {
     p=moves.back();
@@ -334,8 +458,36 @@ void ChessGame::sendMove(int x1,int y1,int x2,int y2) {
 	(y1==1)&&(y2==0))
       promote=1;
   }
-  
-  global.protocol->sendMove(x1,y1,x2,y2,promote);
+
+  if (LocalEdit) {
+    p = new Position();
+    (*p) = *(*cursor);
+
+    q=p->getPiece(x1,y1)&COLOR_MASK;
+    if (p->isMoveLegalCartesian(x1,y1,x2,y2,q,REGULAR)) {
+      p->stdNotationForMove(x1,y1,x2,y2,
+			    promote ?  q | global.promotion->getPiece() : EMPTY,
+			    v, REGULAR);
+      p->setLastMove(v);
+					   
+      p->moveCartesian(x1,y1,x2,y2,REGULAR);
+      if (promote) p->setPiece(x2,y2, q | global.promotion->getPiece() );
+    } else {
+      p->setPiece(x2,y2,p->getPiece(x1,y1));
+      p->setPiece(x1,y1,EMPTY);
+    }
+    
+    z=cursor; z++; if (z!=moves.end()) moves.erase(z,moves.end());
+
+    moves.push_back(p);
+    last_half_move++;
+    cursor=moves.end();
+    cursor--;
+    if (myboard) { myboard->setInfo(2,(*cursor)->getLastMove()); myboard->update(); }
+  } else {
+    if (global.protocol) global.protocol->sendMove(x1,y1,x2,y2,promote);
+  }
+
 }
 
 void ChessGame::flipHint(int flip) {
@@ -378,7 +530,7 @@ int ChessGame::isOver() {
 
 void ChessGame::updateGame(list<Position *> &gamedata) {
   list<Position *>::iterator li;
-  Position *np;
+  Position *np, startpos;
   int hmn=-1;
 
   for(li=moves.begin();li!=moves.end();li++)
@@ -392,6 +544,13 @@ void ChessGame::updateGame(list<Position *> &gamedata) {
     ++hmn;
   }
 
+  // should fix the adding of FEN fields on followed games
+  if (moves.empty() && Variant!=WILD) {
+    np=new Position();
+    moves.push_back(np);
+    hmn=0;
+  }
+
   cursor=moves.end();
   cursor--;
   last_half_move=hmn;
@@ -434,6 +593,7 @@ char * ChessGame::getPlayerString(int index) {
 
 void ChessGame::guessInfoFromPGN() {
   const char *cp;
+  TString t(128);
 
   if (cp=pgn.get("White"))
     strcpy(PlayerName[0],cp);
@@ -448,6 +608,16 @@ void ChessGame::guessInfoFromPGN() {
     if (cp[0]=='1') { result=WHITE_WIN; return; }
     result=UNDEF;
   }
+
+  if (cp=pgn.get("TimeControl")) {
+    t.set((char *) cp);
+    STime = atoi(t.token("+ -/\t"));
+    Inc = atoi(t.token("+ -/\t"));
+  }
+
+  if (cp=pgn.get("Variant")) {
+    Variant = variantFromName((char *)cp);
+  }
 }
 
 void ChessGame::guessPGNFromInfo() {
@@ -489,6 +659,18 @@ void ChessGame::guessPGNFromInfo() {
       rz[strlen(rz)]=Rating[1][i];
   if (strlen(rz)) pgn.set("BlackElo",rz);
 
+  if ( (STime > 0) || (Inc > 0) ) {
+    sprintf(rz,"%d+%d",STime,Inc);
+    pgn.set("TimeControl",rz);
+  }
+
+  if (Variant != REGULAR) {
+    pgn.set("Variant",(char *) variantName(Variant));
+  }
+
+  //  strcpy(rz,Rated?"yes":"no");
+  //  pgn.set("Rated",rz);    
+
   if (over) {
     switch(result) {
     case WHITE_WIN: pgn.set("Result","1-0"); break;
@@ -532,12 +714,14 @@ bool ChessGame::savePGN(char *filename, bool append=false) {
     guessPGNFromInfo();
   pgn.write(f);
 
+  /* TO BE GONE SOON
   if (Variant!=REGULAR)
     fprintf(f,"%%eboard:variant:%s\n",variantName(Variant));
   if (STime > 0)
     fprintf(f,"%%eboard:clock/r:%d:%d:%d\n",STime,Inc,Rated);
   if (strlen(ereason)>5)
     fprintf(f,"%%eboard:clue:%s\n",ereason);
+  */
 
   fprintf(f,"\n");
 
@@ -564,6 +748,12 @@ bool ChessGame::savePGN(char *filename, bool append=false) {
       fprintf(f,"\n");
     }
   }
+
+  if (strlen(ereason) > 5) {
+    if (xp) fprintf(f,"\n");
+    fprintf(f,"{%s} ",ereason);
+  }
+
   cp=pgn.get("Result");
   if (cp)
     fprintf(f,"%s\n\n",cp);
@@ -577,236 +767,398 @@ bool ChessGame::savePGN(char *filename, bool append=false) {
   return true;
 }
 
+// indexonly: do not load the move text now
+bool ChessGame::ParsePgnGame(FILE *f, 
+			     char * filename,
+			     bool indexonly, 
+			     int gameid,
+			     variant v = REGULAR,
+			     ChessGame *updatee = NULL) {
+  /* 
+     ok, now (0.4.x) we do this in a DFA-like manner
+     0  = state 0, initial state (whitespace before movetext)
+
+     PGN header states:
+     1  = found [, consuming PGN field name
+     2  = consuming whitespace between field name and field data
+     3  = found " , consuming field data
+     4  = found ending " , waiting ]
+
+     6  = reading token
+     7  = inside { comment
+     8 = like level 0, but can't have headers
+     9 = got result, waiting end of line (== end of game)
+  */
+
+  int   state    = 0;
+  int   halfmove = 0;
+  piece color    = WHITE;
+
+  list<Position *> gl;
+  list<Position *>::iterator gli;
+  Position * g;
+  ChessGame *game=0;
+
+  ExtPatternMatcher Legacy0, Legacy1, Legacy2, Result;
+  PatternBinder Binder, Binder2;
+
+  char alpha[256], beta[256], omega[256];
+  int ap, bp;
+  char line[512], *p;
+  long pstart;
+  GameResult r;
+
+  static char *whitespace = " \t\n\r";
+  static char *token  =     "1234567890*/KQBNRP@kqbnrp=+#abcdefgh-Ox";
+  static char *digits =     "1234567890";
+  static char *resultset =  "01*";
+
+  Legacy0.set("%%eboard:variant:%s*");
+  Legacy1.set("%%eboard:clock/r:%n:%n:%n*");  
+  Legacy2.set("%%eboard:clue:*");
+  Binder.add(&Legacy0,&Legacy1,&Legacy2,0);
+
+  Result.set("*%N-%N*");
+  Binder2.add(&Result,0);
+
+  pstart=ftell(f);
+  if (!updatee) {
+    game = new ChessGame();
+    game->Variant = v;
+    game->GameNumber = gameid;
+
+    if (indexonly) {
+      game->Loaded=false;
+      strcpy(game->PGNSource,filename);
+      game->SourceOffset = pstart;
+    }
+  }
+
+  omega[0]=0;
+
+  // #define PGN_DEBUG 1
+
+#ifdef PGN_DEBUG
+  cerr << "entering new game\n";
+#endif
+
+  while( fgets(line, 512, f) ) {
+
+#ifdef PGN_DEBUG
+    cerr << "state = " << state << ", current line is: " << line;
+#endif
+
+    // %-comments
+    if (line[0]=='%') {
+      Binder.prepare(line);
+
+      if (Legacy0.match()) {
+	p=Legacy0.getSToken(0);
+	v=variantFromName(p);
+	if (v!=REGULAR)
+	  if (game)
+	    game->Variant=v;
+	continue;
+      } // match Legacy0
+
+      if (Legacy1.match()) {
+	if (game) {
+	  game->STime=atoi(Legacy1.getNToken(0));
+	  game->Inc=atoi(Legacy1.getNToken(1));
+	  game->Rated=atoi(Legacy1.getNToken(2));
+	}
+	continue;
+      } // match Legacy1
+
+      if (Legacy2.match()) {
+	strncpy(omega,Legacy2.getStarToken(0),128);
+	if (omega[strlen(omega)-1]=='\n')
+	  omega[strlen(omega)-1]=0;
+	continue;
+      }
+
+      continue;
+    }
+
+    for(p=line;*p;p++) {
+
+      switch(state) {
+	// whitespace
+      case 0:
+	if (*p == '[')         { state=1; memset(alpha,0,256); ap=0; break; }
+      case 8:
+	if (*p == '{')         { state=7; memset(alpha,0,256); ap=0; break; }
+
+	if (strchr(token, *p)) { 
+	  state=6; 
+	  memset(alpha,0,256);
+	  alpha[0]=*p;
+	  ap=1;
+	  break;
+	}
+	break; // case 0/8
+
+	// header
+      case 1:
+	if (strchr(whitespace, *p)) { state=2; memset(beta,0,256); bp=0; break; }
+	alpha[ap++]=*p;
+	break; // case 1
+
+      case 2:
+	if (*p == '\"') state = 3;
+	break; // case 2
+
+      case 3:
+	if (*p == '\"') state = 4; else beta[bp++]=*p;
+	break; // case 3
+
+      case 4:
+	if (*p == ']') { 
+	  if (game)
+	    game->pgn.set(alpha,beta);
+
+	  if (!indexonly)
+	    if (!strcmp(alpha,"FEN")) {
+	      g=new Position();
+	      g->setFEN(beta);
+	      if (!gl.empty()) gl.clear(); // LEAK
+	      gl.push_back(g);	    
+	    }
+
+	  state = 0;
+	}
+	break; // case 4
+
+	// token
+      case 6:
+	if (strchr(whitespace,*p)) {
+	  Binder2.prepare(alpha);
+
+#ifdef PGN_DEBUG
+	  cerr << "state 6 parsing token " << alpha << endl;
+#endif
+
+	  // move number token (1. , 1...)
+	  if ( strchr(digits, alpha[0]) && strchr(alpha, '.') ) {
+	    state=8; 
+#ifdef PGN_DEBUG
+	    cerr << "v: movenumber\n";
+#endif
+	    break;
+	  }
+
+	  // end game token (1-0, 0-1, 1/2-1/2, *)
+	  if (strchr(resultset, alpha[0]))
+	    if ((alpha[0]=='*') || Result.match() ) {	      
+#ifdef PGN_DEBUG
+	      cerr << "v: end of game\n";
+#endif	      
+
+	      if (!strlen(omega)) strcpy(omega," ");
+	      if (alpha[0]=='*') r=UNDEF; else
+		if (alpha[1]=='/') r=DRAW; else
+		  if (alpha[0]=='1') r=WHITE_WIN; else
+		    r=BLACK_WIN;
+	      if (game) {
+		game->updateGame(gl);
+		game->endGame(omega,r);	    
+		game->guessInfoFromPGN();
+	      } else
+		updatee->updateGame(gl);
+
+	      if (game)
+		global.GameList.push_back(game);
+	      state=9;
+	      break;
+	    }
+
+#ifdef PGN_DEBUG
+	  cerr << "v: move text\n";
+#endif
+	  if (indexonly) { state=8; break; }
+
+	  // move tokens
+	  if (gl.empty()) gl.push_back(new Position());
+	  
+	  g = new Position();
+	  (*g) = *(gl.back());
+	  g->clearAnnotation();
+
+	  g->moveAnyNotation(alpha,color,game ? game->Variant : v);
+	  sprintf(beta,"%d%s %s",1+halfmove/2,
+		  (color==WHITE)?".":". ...",alpha);
+	  g->setLastMove(beta);
+	  gl.push_back(g);
+	  color^=COLOR_MASK;  
+	  halfmove++;
+	  state=8;
+
+	} else { // non whitespace char
+
+	  // ignore move number in PGNs like 1.e4 (no space between . and move)
+	  if ( ap && alpha[ap-1] == '.' )
+	    if (strchr(token,*p)) {
+	      memset(alpha,0,256);
+	      ap=0;
+	    }
+
+	  alpha[ap++]=*p;
+
+	  if (game)
+	    if ( (*p == '@') && (game->Variant == REGULAR) ) {
+	      fseek(f,pstart,SEEK_SET);
+	      gl.clear(); // LEAK
+	      delete game;
+	      return(ParsePgnGame(f,filename,indexonly,gameid,CRAZYHOUSE));
+	    }
+
+	}
+	break; // case 6
+
+	// { comments
+      case 7:
+	if (*p == '}') {
+	  global.annotator.close();
+	  state=8;
+	  break;
+	}
+
+	if (! gl.empty() ) {
+	  if (! global.annotator.isOpen() )
+	    gl.back()->addAnnotation( global.annotator.open() );	    
+	  global.annotator.append(*p >= ' ' ? *p : ' ');
+	}
+
+	break; // case 7
+
+      case 9: // game over, waiting end of line
+	if (*p == '\n') {
+	  for(gli=gl.begin();gli!=gl.end();gli++)
+	    delete(*gli);
+	  gl.clear();
+	  return true;
+	}
+	break; // case 9
+      } // switch state
+
+    } // for p
+
+  } // while fgets
+
+  if (!gl.empty()) gl.clear(); // LEAK
+  if (game) delete game;  
+  return false;
+} // method
+
+bool ChessGame::loadMoves() {
+  FILE *f;
+  char z[512];
+  if (Loaded) return true;
+  f=fopen(PGNSource,"r");
+
+  if (!f) {
+    sprintf(z,"can't load PGN move text from %s (error opening file, errno=%d)",
+	    PGNSource, errno);
+    global.output->append(z,0xc0ff00);
+    return false;
+  }
+
+  if (fseek(f,SourceOffset,SEEK_SET)) {
+    sprintf(z,"can't seek to offset %lu of %s (errno=%d)",
+	    SourceOffset,PGNSource,errno);
+    global.output->append(z,0xc0ff00);
+    fclose(f);
+    return false;
+  }
+  
+  if (!ParsePgnGame(f,PGNSource,false,-1,Variant,this)) {
+    strcpy(z,"error parsing PGN data");
+    global.output->append(z,0xc0ff00);
+    fclose(f);
+    return false;
+  }
+  fclose(f);
+
+  Loaded = true;
+  return true;
+}
+
 void ChessGame::LoadPGN(char *filename) {
   FILE *f;
-  ChessGame *game;
-  char z[512],w[64],mn[32],tmp[64],reas[128];
-  static char *sep=" \t\n\r";
-
-  ExtPatternMatcher PGNHeader, MoveNumber, Result, 
-    VariantHdr, TCHdr, ClueHdr, BMoveNumber;
-  PatternBinder Binder;
-
-  TString t(512);
-  list<Position *> gl;
-  Position *p;
-  char *c,*v,*zzz;
-  piece color;
-  GameResult res;
-  long oops;
-  int inComment=0;
-  int baseid=10000;
-
   UnboundedProgressWindow *pw;
-  int count=0;
+  int nextid,count=0;
+
+  vector<int> g_ids;
+  vector<int>::iterator ii;
+  list<ChessGame *>::iterator gi;
+  bool id_space_empty=false;
 
   f=fopen(filename,"r");
   if (!f)
     return;
 
   //  cerr << "reading " << filename << endl;
-
-  PGNHeader.set("[%s \"*\"]*");
-  MoveNumber.set("%n.");
-  BMoveNumber.set("%n...");
-  Result.set("*%N-%N*");
-  VariantHdr.set("%%eboard:variant:%s*");
-  TCHdr.set("%%eboard:clock/r:%n:%n:%n*");  
-  ClueHdr.set("%%eboard:clue:*");
-
-  Binder.add(&PGNHeader,&VariantHdr,&TCHdr,&ClueHdr,0);
-
-  game=new ChessGame();
-  game->Variant=REGULAR;
-  p=new Position();
-  gl.push_back(p);
-  color=WHITE;
-  oops=ftell(f);
-  reas[0]=0;
-
   pw=new UnboundedProgressWindow("%d games read");
 
-  while(fgets(z,512,f)) {
-    //    cerr << z;
+  // grab a list of existent game ids
+  for(gi=global.GameList.begin();gi!=global.GameList.end();gi++)
+    g_ids.push_back( (*gi)->GameNumber );
+  
+  sort( g_ids.begin(), g_ids.end() );
+  nextid = 10000;
 
-    Binder.prepare(z);
+  for(ii=g_ids.begin(); ii != g_ids.end(); ii++) {
+    if ( *ii >  nextid ) break;
+    if ( *ii == nextid ) ++nextid;
+  }
 
-    if (TCHdr.match()) {
-      game->STime=atoi(TCHdr.getNToken(0));
-      game->Inc=atoi(TCHdr.getNToken(1));
-      game->Rated=atoi(TCHdr.getNToken(2));
-      continue;
-    }
+  if ( ii == g_ids.end() )
+    id_space_empty = true;
 
-    if (ClueHdr.match()) {
-      strncpy(reas,ClueHdr.getStarToken(0),128);
-      if (reas[strlen(reas)-1]=='\n')
-	reas[strlen(reas)-1]=0;
-      continue;
-    }
+  while(!feof(f)) {
 
-    if (VariantHdr.match()) {
-      variant vr;
-      vr=REGULAR;
-      c=VariantHdr.getSToken(0);
-      if (!strcmp(c,variantName(CRAZYHOUSE))) vr=CRAZYHOUSE;
-      else
-	if (!strcmp(c,variantName(BUGHOUSE)))   vr=BUGHOUSE;
-	else
-	  if (!strcmp(c,variantName(WILD)))       vr=WILD;
-	  else
-	    if (!strcmp(c,variantName(SUICIDE)))    vr=SUICIDE;
-	    else
-	      if (!strcmp(c,variantName(LOSERS)))     vr=LOSERS;
-	      else
-		if (!strcmp(c,variantName(GIVEAWAY)))   vr=GIVEAWAY;
-      if (vr!=REGULAR)
-	game->Variant=vr;
-      continue;
-    }
+    if (ParsePgnGame(f,filename,true,nextid)) {
 
-    if (z[0]=='%')
-      continue;
-    
-    if (PGNHeader.match()) {
-      strncpy(tmp,PGNHeader.getSToken(0),64);
-      game->pgn.set((const char *)tmp, PGNHeader.getStarToken(0));
-
-      if (!strcmp(tmp,"FEN"))
-	p->setFEN(game->pgn.get("FEN"));
-
-      continue;
-    }
-
-    inComment=StripPGNComments(z,inComment);
-
-    t.set(z);
-    while( ( c=t.token(sep) )!= 0 ) {
-      //      cerr << "token(" << c << ")\n";
-      if ((c[0]>='0')&&(c[0]<='9')&&(MoveNumber.match(c))) { strcpy(mn,c); continue; }
-      if (strchr("*01",c[0])||c[1]=='*')
-	if ((c[0]=='*')||(c[1]=='*')||(Result.match(c))) {
-	  game->updateGame(gl);
-	  if ((c[0]=='*')||(c[1]=='*')) {
-	    res=UNDEF;
-	  } else {
-	    v=Result.getNToken(0);
-	    switch(v[0]) {
-	    case '0': res=BLACK_WIN; break;
-	    case '1': res=WHITE_WIN; break;
-	    case '2': res=DRAW; break;
-	    default: res=UNDEF;
-	    }
-	  }
-	  if (!strlen(reas)) strcpy(reas," ");
-	  game->endGame(reas,res);
-	  game->guessInfoFromPGN();
-	  game->GameNumber=baseid=global.nextFreeGameId(baseid);
-	  ++baseid;
-	  global.appendGame(game,false);
-	  
-	  // prepare for next game
-	  while(!gl.empty()) {
-	    delete(gl.front());
-	    gl.erase(gl.begin());
-	  }
-	  game=new ChessGame();
-	  game->Variant=REGULAR;
-	  p=new Position();
-	  gl.push_back(p);
-	  color=WHITE;
-	  
-	  ++count;
-	  if (! (count%5) )
-	    pw->setProgress(count);
-	  oops=ftell(f);
-	  reas[0]=0;
-	  break;
+      ++count;
+      ++nextid;
+      if (!id_space_empty) {
+	for( ; ii != g_ids.end(); ii++) {
+	  if ( *ii >  nextid ) break;
+	  if ( *ii == nextid ) ++nextid;
 	}
-
-      if (BMoveNumber.match(c)) // skip any 14... tokens
-	continue;
-
-      p=new Position();
-      (*p)=*(gl.back());
-      
-      // crazyhouse detection hack
-      if ((c[1]=='@')&&(game->Variant==REGULAR)) {
-	// oh-oh, this is crazyhouse, all game data
-	// is oopsed, dump everything and rewind file
-
-	while(!gl.empty()) {
-	  delete(gl.front());
-	  gl.erase(gl.begin());
-	}
-	delete game;
-
-	game=new ChessGame();
-	game->Variant=CRAZYHOUSE;
-	p=new Position();
-	gl.push_back(p);
-	color=WHITE;
-	fseek(f,oops,SEEK_SET);
-	reas[0]=0;
-	break; // process line
+	if ( ii == g_ids.end() ) id_space_empty = true;
       }
 
-      p->moveAnyNotation(c,color,game->Variant);
-      sprintf(w,"%s%s%s",mn,(color==WHITE)?" ":" ... ",c);
-      p->setLastMove(w);
-      gl.push_back(p);
-      color^=COLOR_MASK;      
-    } // process line
+    } // parse success
 
-  } // get more lines
+    if (! (count%5) ) pw->setProgress(count);
+  }
 
   fclose(f);
-
-  // leftovers
-  while(!gl.empty()) {
-    delete(gl.front());
-    gl.erase(gl.begin());
-  }
-  delete game;
+  g_ids.clear();
   delete pw;
 }
 
-int ChessGame::StripPGNComments(char *z, int incomment) {
-  char aux[512];
-  char *p,*q;
-  int ic;
-
-  ic=incomment;
-  strcpy(aux,z);
-  memset(z,0,512);
- 
-  p=aux;
-  q=z;
-
-  for(;*p;p++) {
-    if (ic) { // we're inside a comment
-
-      if (*p == '}')
-	ic=0;
-
-    } else { // we're not inside a comment
-
-      if (*p == '{') {
-	ic=1;
-      } else {
-	*(q++)=*p;
-      }
-
-    }
-  }
-
-  return ic;
-}
-
 GameResult ChessGame::getResult() {
   return(over?result:UNDEF);
 }
 
+variant ChessGame::variantFromName(char *p) {
+  variant v=REGULAR;
+  if (!strcmp(p,variantName(CRAZYHOUSE))) v=CRAZYHOUSE;
+  else
+    if (!strcmp(p,variantName(BUGHOUSE)))   v=BUGHOUSE;
+    else
+      if (!strcmp(p,variantName(WILD)))       v=WILD;
+      else
+	if (!strcmp(p,variantName(SUICIDE)))    v=SUICIDE;
+	else
+	  if (!strcmp(p,variantName(LOSERS)))     v=LOSERS;
+	  else
+	    if (!strcmp(p,variantName(GIVEAWAY)))   v=GIVEAWAY;
+  return v;
+}
+
 const char * ChessGame::variantName(variant v) {
   switch(v) {
   case REGULAR:    return("chess");
