@@ -1,4 +1,4 @@
-/* $Id: chess.h,v 1.28 2001/10/24 11:10:22 bergo Exp $ */
+/* $Id: chess.h,v 1.32 2002/02/10 13:43:40 bergo Exp $ */
 
 /*
 
@@ -70,6 +70,7 @@ class ChessGame : public MoveListListener {
   ChessGame();
   ChessGame(int _number,int _tyme,int _inc, int _rated,variant _variant,
 	    char *p1,char *p2);
+  ChessGame(ChessGame *src);
   ~ChessGame();
 
   int operator==(int gnum);
@@ -112,15 +113,26 @@ class ChessGame : public MoveListListener {
 
   void retreat(int nmoves);
   static void LoadPGN(char *filename);
-  static int  StripPGNComments(char *z,int incomment);
+  static bool ParsePgnGame(FILE *f,
+			   char * filename,
+			   bool indexonly, 
+			   int gameid,
+			   variant v = REGULAR,
+			   ChessGame *updatee = NULL);
   bool savePGN(char *filename, bool append=false);
 
+  bool loadMoves();
+
   bool isFresh();
 
   void guessInfoFromPGN();
   void guessPGNFromInfo();
 
+  void editEmpty();
+  void editStartPos();
+
   static const char *variantName(variant v);
+  static variant variantFromName(char *p);
 
   int     GameNumber;
   int     STime;
@@ -132,6 +144,12 @@ class ChessGame : public MoveListListener {
   piece   MyColor;
   int     StopClock; // for examined games
 
+  bool    LocalEdit;
+
+  bool    Loaded;
+  char    PGNSource[256];
+  long    SourceOffset;
+
   PGNheader pgn;
 
   char info0[64];
@@ -141,6 +159,9 @@ class ChessGame : public MoveListListener {
 
  private:
   void showResult();
+  void fixExamineZigZag(Position &suspect);
+  void junkMoves(list<Position *>::iterator from,
+		 list<Position *>::iterator to);
 
   list<Position *> moves;
   list<Position *>::iterator cursor;
