@@ -1,10 +1,10 @@
-/* $Id: dlg_gamelist.cc,v 1.13 2001/10/24 11:10:22 bergo Exp $ */
+/* $Id: dlg_gamelist.cc,v 1.17 2002/03/17 04:53:36 bergo Exp $ */
 
 /*
 
     eboard - chess client
     http://eboard.sourceforge.net
-    Copyright (C) 2000-2001 Felipe Paulo Guazzi Bergo
+    Copyright (C) 2000-2002 Felipe Paulo Guazzi Bergo
     bergo@seul.org
 
     This program is free software; you can redistribute it and/or modify
@@ -35,6 +35,14 @@
 #include "icon-ads.xpm"
 #include "icon-local.xpm"
 
+#include "goc_open.xpm"
+#include "goc_save.xpm"
+#include "goc_refresh.xpm"
+#include "goc_discardall.xpm"
+#include "goc_display.xpm"
+#include "goc_discard.xpm"
+#include "goc_edit.xpm"
+
 GameListDialog::GameListDialog(GameListListener *someone) {
   GtkWidget *sw,*v,*bh;
   GtkStyle *style;
@@ -206,7 +214,7 @@ gamelist_unselect  (GtkCList *cl, gint row, gint column, GdkEventButton *eb,
 // ------------- client-side game list
 
 StockListDialog::StockListDialog(StockListListener *someone) {
-  GtkWidget *v,*sw,*bh,*ol;
+  GtkWidget *v,*sw,*h,*left;
   GdkFont *fixed;
   GtkStyle *style;
   int i;
@@ -222,8 +230,37 @@ StockListDialog::StockListDialog(StockListListener *someone) {
   gtk_container_set_border_width(GTK_CONTAINER(widget),4);
   gtk_widget_realize(widget);
 
+  h=gtk_hbox_new(FALSE,0);
+  gtk_container_add(GTK_CONTAINER(widget),h);
+
+  left=gtk_vbox_new(FALSE,0);
+  gtk_box_pack_start(GTK_BOX(h),left,FALSE,TRUE,0);
+
+  b[3]=PixButton(goc_open_xpm,"Load PGN...");
+  gtk_box_pack_start(GTK_BOX(left),b[3],FALSE,TRUE,0);
+
+  b[0]=PixButton(goc_refresh_xpm,"Refresh List");
+  gtk_box_pack_start(GTK_BOX(left),b[0],FALSE,TRUE,0);
+
+  b[6]=PixButton(goc_discardall_xpm,"Discard All");
+  gtk_box_pack_start(GTK_BOX(left),b[6],FALSE,TRUE,0);
+
+  b[1]=PixButton(goc_display_xpm,"Display Game");
+  gtk_box_pack_start(GTK_BOX(left),b[1],FALSE,TRUE,0);
+
+  b[4]=PixButton(goc_save_xpm,"Save Game...");
+  gtk_box_pack_start(GTK_BOX(left),b[4],FALSE,TRUE,0);
+
+  b[2]=PixButton(goc_discard_xpm,"Discard Game");
+  gtk_box_pack_start(GTK_BOX(left),b[2],FALSE,TRUE,0);
+
+  b[5]=PixButton(goc_edit_xpm,"Edit Game Info");
+  gtk_box_pack_start(GTK_BOX(left),b[5],FALSE,TRUE,0);
+
+  // -------
+
   v=gtk_vbox_new(FALSE,0);
-  gtk_container_add(GTK_CONTAINER(widget),v);
+  gtk_box_pack_start(GTK_BOX(h),v,TRUE,TRUE,0);
 
   sw=gtk_scrolled_window_new(0,0);
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw),
@@ -240,7 +277,7 @@ StockListDialog::StockListDialog(StockListListener *someone) {
   clist=gtk_clist_new(6);
   gtk_clist_set_shadow_type(GTK_CLIST(clist),GTK_SHADOW_IN);
   gtk_clist_set_selection_mode(GTK_CLIST(clist),GTK_SELECTION_SINGLE);
-  gtk_clist_set_column_title(GTK_CLIST(clist),0,"#");
+  gtk_clist_set_column_title(GTK_CLIST(clist),0,"Game #");
   gtk_clist_set_column_title(GTK_CLIST(clist),1,"Displayed");
   gtk_clist_set_column_title(GTK_CLIST(clist),2,"White");
   gtk_clist_set_column_title(GTK_CLIST(clist),3,"Black");
@@ -260,34 +297,8 @@ StockListDialog::StockListDialog(StockListListener *someone) {
     gtk_clist_set_column_min_width(GTK_CLIST(clist),i,64);
   gtk_clist_columns_autosize(GTK_CLIST(clist));
 
-  bh=gtk_table_new(TRUE,2,5);
-  gtk_box_pack_start(GTK_BOX(v),bh,FALSE,FALSE,0);
-
-  b[0]=gtk_button_new_with_label("Refresh List");
-  b[1]=gtk_button_new_with_label("Display");
-  b[2]=gtk_button_new_with_label("Discard");
-  b[3]=gtk_button_new_with_label("Load PGN...");
-  b[4]=gtk_button_new_with_label("Save as PGN...");
-  b[5]=gtk_button_new_with_label("Edit PGN Info");
-  b[6]=gtk_button_new_with_label("Discard All");
-
-  ol=gtk_label_new("Selected Game:");
-
-  gtk_label_set_justify(GTK_LABEL(ol),GTK_JUSTIFY_RIGHT);
-
-  gtk_table_attach_defaults(GTK_TABLE(bh),ol,0,1,0,1);
-  gtk_table_attach_defaults(GTK_TABLE(bh),b[1],1,2,0,1);
-  gtk_table_attach_defaults(GTK_TABLE(bh),b[2],2,3,0,1);
-  gtk_table_attach_defaults(GTK_TABLE(bh),b[4],3,4,0,1);
-  gtk_table_attach_defaults(GTK_TABLE(bh),b[5],4,5,0,1);
-
-  gtk_table_attach_defaults(GTK_TABLE(bh),b[0],1,2,1,2);
-  gtk_table_attach_defaults(GTK_TABLE(bh),b[6],2,3,1,2);
-  gtk_table_attach_defaults(GTK_TABLE(bh),b[3],3,4,1,2);
-
   for(i=0;i<7;i++)
     gtk_widget_show(b[i]);
-  gtk_widget_show(ol);
 
   gtk_widget_set_sensitive(b[0],TRUE);
   gtk_widget_set_sensitive(b[1],FALSE);
@@ -299,11 +310,13 @@ StockListDialog::StockListDialog(StockListListener *someone) {
 
   setIcon(icon_local_xpm,"Local");
 
-  gtk_widget_show(bh);
   gtk_widget_show(clist);
   gtk_widget_show(sw);
   gtk_widget_show(v);
 
+  gtk_widget_show(left);
+  gtk_widget_show(h);
+
   gtk_signal_connect(GTK_OBJECT(widget),"delete_event",
 		     GTK_SIGNAL_FUNC(gamelist_delete),(gpointer)(&canclose));
   gtk_signal_connect(GTK_OBJECT(widget),"destroy",
@@ -383,7 +396,8 @@ void StockListDialog::open() {
   cg=global.getGame(getSelCol0());
   if (!cg)
     return;
-  
+
+  if (!cg->loadMoves()) return;
   cg->protodata[1]=1;
 
   b=new Board();
