@@ -1,10 +1,10 @@
-/* $Id: dlg_prefs.cc,v 1.15 2001/09/06 04:44:40 bergo Exp $ */
+/* $Id: dlg_prefs.cc,v 1.18 2002/03/17 04:53:36 bergo Exp $ */
 
 /*
 
     eboard - chess client
     http://eboard.sourceforge.net
-    Copyright (C) 2000-2001 Felipe Paulo Guazzi Bergo
+    Copyright (C) 2000-2002 Felipe Paulo Guazzi Bergo
     bergo@seul.org
 
     This program is free software; you can redistribute it and/or modify
@@ -38,20 +38,32 @@ typedef void (*sigfunc)(int);
 
 PreferencesDialog::PreferencesDialog() : ModalDialog("Preferences") {
   GtkWidget *v,*hs,*ok,*apply,*cancel,*bhb,*nb;
-  GtkWidget *uil,*uiv,*uih,*tfr,*tr[4],*tv,*uih2,*uih3;
+
+  // gui
+  GtkWidget *uil,*uiv,*uih,*tfr,*tr[4],*tv,*uih2,*uih3,*uih4;
   GSList *tl;
 
+  // -- fonts
   GtkWidget *fl, *fv, *ft, *fn[4], *fb[4], *frv;
+
+  // -- ics
   GtkWidget *icv, *icl, *ich1, *icl1, *ifr1, *ifr2, *ifv1, *ifv2;
-  GtkWidget *sv, *sl, *sf[3], *sh[3], *sb[3][2];
+
+  // -- sound
+  GtkWidget *sv,*sl,*zl,*zf,*zv,*zd,*zt,*ze,*zh;
+  static char *zcaption[]={"Opponent Moved","Draw Offered","Private Tell","ICS Challenge"};
+  char **zp;
+
+  // -- autosave
   GtkWidget *gv, *gl, *gll, *gh;
+
+  // -- colors
   GtkWidget *tcl, *tcv, *tch, *tcv2, *tcdef;
   int i;
   char z[64];
 
-  sndcopy[0]=global.sndevents[0];
-  sndcopy[1]=global.sndevents[1];
-  sndcopy[2]=global.sndevents[2];
+  for(i=0;i<4;i++)
+    sndcopy[i]=global.sndevents[i];
 
   gtk_window_set_default_size(GTK_WINDOW(widget),450,300);
 
@@ -104,22 +116,30 @@ PreferencesDialog::PreferencesDialog() : ModalDialog("Preferences") {
 
   uih2=gtk_hbox_new(FALSE,4);
   uih3=gtk_hbox_new(FALSE,4);
+  uih4=gtk_hbox_new(FALSE,4);
   aapieceb=gtk_check_button_new_with_label("Antialias piece images");
   gtk_box_pack_start(GTK_BOX(uih2),aapieceb,FALSE,FALSE,0);
 
   dhsb=gtk_check_button_new_with_label("Graphic representation of crazy/bughouse stock");
   gtk_box_pack_start(GTK_BOX(uih3),dhsb,FALSE,FALSE,0);
 
+  aqbar=gtk_check_button_new_with_label("Show shortcut buttons below board");
+  gtk_box_pack_start(GTK_BOX(uih4),aqbar,FALSE,FALSE,0);
+
   gtk_box_pack_start(GTK_BOX(uiv),uih2,FALSE,FALSE,4);
   gtk_box_pack_start(GTK_BOX(uiv),uih3,FALSE,FALSE,4);
+  gtk_box_pack_start(GTK_BOX(uiv),uih4,FALSE,FALSE,4);
 
   gtk_widget_show(aapieceb);
   gtk_widget_show(dhsb);
+  gtk_widget_show(aqbar);
   gtk_widget_show(uih2);
   gtk_widget_show(uih3);
+  gtk_widget_show(uih4);
 
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(aapieceb), global.AntialiasPieces?1:0);
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(dhsb), global.DrawHouseStock?1:0);
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(aqbar), global.ShowQuickbar?1:0);
 
   gtk_widget_show(tv);
   gtk_widget_show(tfr);
@@ -333,36 +353,63 @@ PreferencesDialog::PreferencesDialog() : ModalDialog("Preferences") {
   gtk_container_set_border_width(GTK_CONTAINER(sv),6);
   sl=gtk_label_new("Sounds");
 
-  for(i=0;i<3;i++) {
-    sf[i]=gtk_frame_new((i==0)?"Event: Opponent Moved":(i==1)?"Event: Draw Offers":"Event: Private Tells");
-    gtk_frame_set_shadow_type(GTK_FRAME(sf[i]),GTK_SHADOW_ETCHED_IN);
+  zf=gtk_frame_new("Sound Events");
+  gtk_frame_set_shadow_type(GTK_FRAME(zf),GTK_SHADOW_ETCHED_IN);
 
-    sh[i]=gtk_hbox_new(FALSE,4);
-    gtk_container_set_border_width(GTK_CONTAINER(sh[i]),4);
-    snddesc[i]=gtk_label_new(sndcopy[i].getDescription());
-    sb[i][0]=gtk_button_new_with_label(" Test ");
-    sb[i][1]=gtk_button_new_with_label(" Edit... ");
-    gtk_box_pack_start(GTK_BOX(sh[i]),snddesc[i],FALSE,TRUE,4);
-    gtk_box_pack_end(GTK_BOX(sh[i]),sb[i][1],FALSE,TRUE,4);
-    gtk_box_pack_end(GTK_BOX(sh[i]),sb[i][0],FALSE,TRUE,4);
-    gtk_container_add(GTK_CONTAINER(sf[i]),sh[i]);
-    gtk_box_pack_start(GTK_BOX(sv),sf[i],FALSE,FALSE,4);
-    gtk_widget_show(sh[i]);
-    gtk_widget_show(snddesc[i]);
-    gtk_widget_show(sb[i][0]);
-    gtk_widget_show(sb[i][1]);
-    gtk_widget_show(sb[i][1]);
-    gtk_widget_show(sf[i]);
+  zv=gtk_vbox_new(FALSE,4);
+  gtk_container_set_border_width(GTK_CONTAINER(zv), 4);
 
-    gtk_signal_connect(GTK_OBJECT(sb[i][0]),"clicked",
-		       GTK_SIGNAL_FUNC(prefs_sndtest),(gpointer)(&sndcopy[i]));
+  gtk_box_pack_start(GTK_BOX(sv),zf,TRUE,TRUE,2);
+  gtk_container_add(GTK_CONTAINER(zf),zv);
+
+  zl=gtk_clist_new(1);
+  gtk_clist_column_titles_hide(GTK_CLIST(zl));
+  gtk_box_pack_start(GTK_BOX(zv),zl,TRUE,TRUE,4);
+
+  for(i=0;i<4;i++) {
+    zp=&(zcaption[i]);
+    gtk_clist_append(GTK_CLIST(zl),(gchar **) zp);
   }
-  gtk_signal_connect(GTK_OBJECT(sb[0][1]),"clicked",
-		     GTK_SIGNAL_FUNC(prefs_sndedit0),(gpointer)(this));
-  gtk_signal_connect(GTK_OBJECT(sb[1][1]),"clicked",
-		     GTK_SIGNAL_FUNC(prefs_sndedit1),(gpointer)(this));
-  gtk_signal_connect(GTK_OBJECT(sb[2][1]),"clicked",
-		     GTK_SIGNAL_FUNC(prefs_sndedit2),(gpointer)(this));
+
+  zd=gtk_label_new(" ");
+  gtk_box_pack_start(GTK_BOX(zv),zd,FALSE,TRUE,4);
+
+  zh=gtk_hbutton_box_new();
+  gtk_button_box_set_layout(GTK_BUTTON_BOX(zh), GTK_BUTTONBOX_END);
+  gtk_button_box_set_spacing(GTK_BUTTON_BOX(zh), 5);
+
+  zt=gtk_button_new_with_label("Test");
+  ze=gtk_button_new_with_label("Edit");
+
+  gtk_box_pack_start(GTK_BOX(zh),zt,TRUE,TRUE,2);
+  gtk_box_pack_start(GTK_BOX(zh),ze,TRUE,TRUE,2);
+  gtk_box_pack_start(GTK_BOX(zv),zh,FALSE,TRUE,2);
+
+  gtk_widget_set_sensitive(zt, FALSE);
+  gtk_widget_set_sensitive(ze, FALSE);
+
+  gtk_widget_show(ze);
+  gtk_widget_show(zt);
+  gtk_widget_show(zh);
+  gtk_widget_show(zd);
+  gtk_widget_show(zl);
+  gtk_widget_show(zv);
+  gtk_widget_show(zf);
+
+  snddesc=zd;
+  sndb[0]=zt;
+  sndb[1]=ze;
+  selected_sound_event=-1;
+  
+  gtk_signal_connect(GTK_OBJECT(zt),"clicked",
+		     GTK_SIGNAL_FUNC(prefs_sndtest),(gpointer) this);
+  gtk_signal_connect(GTK_OBJECT(ze),"clicked",
+		     GTK_SIGNAL_FUNC(prefs_sndedit),(gpointer) this);
+
+  gtk_signal_connect(GTK_OBJECT(zl),"select_row",
+		     GTK_SIGNAL_FUNC(prefs_sndsel),(gpointer) this);
+  gtk_signal_connect(GTK_OBJECT(zl),"unselect_row",
+		     GTK_SIGNAL_FUNC(prefs_sndunsel),(gpointer) this);
 
   gtk_widget_show(sv);
   gtk_widget_show(sl);
@@ -503,6 +550,7 @@ void PreferencesDialog::Apply() {
   ApplyCheckBox(plainb, &global.PlainSquares, &changed, &psetchg);
   ApplyCheckBox(aapieceb, &global.AntialiasPieces, &changed, &psetchg);
   ApplyCheckBox(dhsb, &global.DrawHouseStock, &changed, 0);
+  ApplyCheckBox(aqbar, &global.ShowQuickbar, &changed, 0);
   ApplyCheckBox(showratb, &global.ShowRating, &changed, 0);
   ApplyCheckBox(autologb, &global.FicsAutoLogin, &changed, 0);
   ApplyCheckBox(sgb, &global.IcsSeekGraph, &changed, 0);
@@ -544,7 +592,7 @@ void PreferencesDialog::Apply() {
     global.updateScrollBacks();
   }
 
-  for(i=0;i<3;i++)
+  for(i=0;i<4;i++)
     if (sndcopy[i] != global.sndevents[i]) {
       global.sndevents[i]=sndcopy[i];
       changed=1;
@@ -565,6 +613,9 @@ void PreferencesDialog::Apply() {
     global.output->updateFont();
   if (tbg)
     global.output->setBackground(global.Colors.Background);
+
+  if (changed)
+    global.qbcontainer->update();
    
   if (changed)
     global.writeRC();
@@ -808,35 +859,45 @@ void PreferencesDialog::ResolveFont(char *src,char *dest) {
 // sounds
 
 void prefs_sndtest(GtkWidget *w,gpointer data) {
-  SoundEvent *se;
-  se=(SoundEvent *)data;
-  se->safePlay();
-}
-
-void prefs_sndedit0(GtkWidget *w,gpointer data) {
   PreferencesDialog *me;
   me=(PreferencesDialog *)data;
-  me->sndcopy[0].edit(me);
+  me->sndcopy[me->selected_sound_event].safePlay();
 }
 
-void prefs_sndedit1(GtkWidget *w,gpointer data) {
+void prefs_sndedit(GtkWidget *w,gpointer data) {
   PreferencesDialog *me;
   me=(PreferencesDialog *)data;
-  me->sndcopy[1].edit(me);
-}
-
-void prefs_sndedit2(GtkWidget *w,gpointer data) {
-  PreferencesDialog *me;
-  me=(PreferencesDialog *)data;
-  me->sndcopy[2].edit(me);
+  me->sndcopy[me->selected_sound_event].edit(me);
 }
 
 void PreferencesDialog::SoundEventChanged() {
-  gtk_label_set_text(GTK_LABEL(snddesc[0]),sndcopy[0].getDescription());
-  gtk_label_set_text(GTK_LABEL(snddesc[1]),sndcopy[1].getDescription());
-  gtk_label_set_text(GTK_LABEL(snddesc[2]),sndcopy[2].getDescription());
-  gtk_widget_queue_resize(snddesc[0]);
-  gtk_widget_queue_resize(snddesc[1]);
-  gtk_widget_queue_resize(snddesc[2]);
+  gboolean sens;
+  int i;
+ 
+  if (selected_sound_event < 0) {
+    gtk_label_set_text(GTK_LABEL(snddesc)," ");
+    sens=FALSE;
+  } else {
+    gtk_label_set_text(GTK_LABEL(snddesc),sndcopy[selected_sound_event].getDescription());
+    sens=TRUE;
+  }
+
+  for(i=0;i<2;i++)
+    gtk_widget_set_sensitive(sndb[i],sens);
+
+  gtk_widget_queue_resize(snddesc);
 }
 
+void prefs_sndsel(GtkCList *c,gint row, gint col, GdkEventButton *eb, gpointer data) {
+  PreferencesDialog *me;
+  me=(PreferencesDialog *)data;
+  me->selected_sound_event=row;
+  me->SoundEventChanged();
+}
+
+void prefs_sndunsel(GtkCList *c,gint row, gint col, GdkEventButton *eb, gpointer data) {
+  PreferencesDialog *me;
+  me=(PreferencesDialog *)data;
+  me->selected_sound_event=-1;
+  me->SoundEventChanged();
+}
