@@ -1,4 +1,4 @@
-/* $Id: eboard.h,v 1.20 2001/07/31 04:14:17 bergo Exp $ */
+/* $Id: eboard.h,v 1.23 2002/03/12 02:03:23 bergo Exp $ */
 
 /*
 
@@ -116,6 +116,16 @@ class PaneChangeListener {
   virtual void paneChanged(int pgseq,int pgid)=0;
 };
 
+class SigChildHandler {
+ public:
+  virtual void ZombieNotification(int pid)=0;
+};
+
+class BookmarkListener {
+ public:
+  virtual void updateBookmarks()=0;
+};
+
 typedef enum {
   REGULAR,
   CRAZYHOUSE,
@@ -149,5 +159,11 @@ typedef enum {
 #define DEFAULT_FONT_INFO "-*-helvetica-medium-r-normal--12-*-*-*-*-*-*"
 #define DEFAULT_FONT_CONS "-misc-fixed-medium-r-normal--14-*-*-*-*-*-*"
 
+typedef enum {
+  EF_PlayerFont,
+  EF_ClockFont,
+  EF_InfoFont
+} EboardFont;
+
 #endif
 
