@@ -1,10 +1,10 @@
-/* $Id: global.cc,v 1.40 2001/09/30 17:48:23 bergo Exp $ */
+/* $Id: global.cc,v 1.48 2002/03/17 20:51:47 bergo Exp $ */
 
 /*
 
     eboard - chess client
     http://eboard.sourceforge.net
-    Copyright (C) 2000-2001 Felipe Paulo Guazzi Bergo
+    Copyright (C) 2000-2002 Felipe Paulo Guazzi Bergo
     bergo@seul.org
 
     This program is free software; you can redistribute it and/or modify
@@ -32,6 +32,7 @@
 #include <dirent.h>
 #include <sys/stat.h>
 #include <sys/types.h>
+#include <sys/wait.h>
 #include <time.h>
 #include <signal.h>
 
@@ -44,6 +45,7 @@
 #include "tstring.h"
 #include "notebook.h"
 #include "board.h"
+#include "quickbar.h"
 
 Global global;
 
@@ -58,6 +60,11 @@ Global::Global() {
   ebook=0;
   skgraph=0;
   inputhistory=0;
+  bmlistener=0;
+  qbcontainer=0;
+  quickbar=0;
+
+  LastScratch = 0;
 
   Quitting=0;
   Version=VERSION;
@@ -107,12 +114,18 @@ Global::Global() {
   SplitChannels=0;
   ChannelsToConsoleToo=0;
 
+  ShowQuickbar=1;
+
   sndevents[1].Pitch=650;
   sndevents[1].Duration=350;
 
   sndevents[2].Pitch=900;
   sndevents[2].Duration=80;
 
+  sndevents[3].Pitch=444;
+  sndevents[3].Duration=40;
+  sndevents[3].Count=3;
+
   strcpy(ClockFont,DEFAULT_FONT_CLOK);
   strcpy(PlayerFont,DEFAULT_FONT_PLYR);
   strcpy(InfoFont,DEFAULT_FONT_INFO);
@@ -155,7 +168,17 @@ Global::Global() {
   RCKeys.push_back("TerminalColors");  //34
   RCKeys.push_back("DesktopState");
   RCKeys.push_back("DesktopStateDC");  //36
+  RCKeys.push_back("ShowQuickbar");
+  RCKeys.push_back("QuickbarButton");  //38
 
+  PopupHelp = false;
+}
+
+void Global::dropQuickbarButtons() {
+  int i;
+  for(i=0;i<QuickbarButtons.size();i++)
+    delete(QuickbarButtons[i]);
+  QuickbarButtons.clear();
 }
 
 void Global::clearDupes(ChessGame *cg) {
@@ -165,6 +188,15 @@ void Global::clearDupes(ChessGame *cg) {
       renumberGame(*gi,nextFreeGameId(8000));
 }
 
+void Global::deleteGame(ChessGame *cg) {
+  list<ChessGame *>::iterator gi;
+  for(gi=GameList.begin();gi!=GameList.end();gi++)
+    if ( (*gi) == cg ) {
+      GameList.erase(gi);
+      return;
+    }
+}
+
 void Global::appendGame(ChessGame *cg,bool RenumberDupes=true) {
   if (RenumberDupes) clearDupes(cg);
   GameList.push_back(cg);
@@ -179,7 +211,7 @@ void Global::renumberGame(ChessGame *cg,int id) {
   oldid=cg->GameNumber;
   cg->GameNumber=id;
   // renumber notebook references
-  if (ebook) ebook->renumberPage(oldid,id);  
+  if (ebook) ebook->renumberPage(oldid,id);
 }
 
 void Global::removeBoard(Board *b) {
@@ -216,6 +248,13 @@ void Global::ensureDirectories() {
     return;
   }
   sprintf(z,"%s/.eboard",home);
+
+  tdir=opendir(z);
+  if (tdir==NULL)
+    PopupHelp = true;
+  else
+    closedir(tdir);
+    
   if (createDir(z)) return;
   sprintf(z,"%s/.eboard/craftylog",home);
   createDir(z);
@@ -246,6 +285,8 @@ void Global::readRC() {
   FILE *f;
   HostBookmark *hbm;
   int i;
+  QButton *qb;
+
   sprintf(z,"%s/.eboard/eboard.conf",getenv("HOME"));
 
   f=fopen(z,"r");
@@ -298,7 +339,7 @@ void Global::readRC() {
     // sound events
     if (!strcmp(p,RCKeys[17])) {
       i=atoi(t.token(sep));
-      if (i<3)
+      if (i<4)
 	sndevents[i].read(t);
       continue;
     }
@@ -327,6 +368,15 @@ void Global::readRC() {
     if (!strcmp(p,RCKeys[34])) { Colors.read(t); continue; }
     if (!strcmp(p,RCKeys[35])) { Desk.read(t); continue; }
     if (!strcmp(p,RCKeys[36])) { Desk.readConsole(t); continue; }
+    if (!strcmp(p,RCKeys[37])) { ShowQuickbar=atoi(t.token(sep)); continue; }
+    if (!strcmp(p,RCKeys[38])) { 
+      qb=new QButton();
+      qb->icon=atoi(t.token("\n\r\t:"));
+      qb->caption=t.token("\n\r\t:");
+      qb->command=t.token("\n\r\t:");
+      QuickbarButtons.push_back(qb);
+    }
+
   }
   fclose(f);  
 }
@@ -367,7 +417,7 @@ void Global::writeRC() {
   fprintf(f,"%s::%d\n",RCKeys[15],FicsAutoLogin);
   fprintf(f,"%s::%d\n",RCKeys[16],BeepWhenOppMoves);
 
-  for(i=0;i<3;i++)
+  for(i=0;i<4;i++)
     sndevents[i].write(f,RCKeys[17],i);
 
   fprintf(f,"%s::%d\n",RCKeys[18],EnableSounds);
@@ -390,6 +440,13 @@ void Global::writeRC() {
   fprintf(f,"%s::",RCKeys[34]); Colors.write(f);
   fprintf(f,"%s::",RCKeys[35]); Desk.write(f);
   Desk.writeConsoles(f,RCKeys[36]);
+  fprintf(f,"%s::%d\n",RCKeys[37],ShowQuickbar);
+  for(i=0;i<QuickbarButtons.size();i++)
+    fprintf(f,"%s::%d:%s:%s\n",RCKeys[38], 
+	    QuickbarButtons[i]->icon,
+	    QuickbarButtons[i]->caption.c_str(),
+	    QuickbarButtons[i]->command.c_str());
+
   fclose(f);
 }
 
@@ -492,6 +549,11 @@ void Global::privatelyTold() {
     sndevents[2].safePlay();
 }
 
+void Global::challenged() {
+  if (EnableSounds)
+    sndevents[3].safePlay();
+}
+
 void Global::repaintAllBoards() {
   for(BLi=BoardList.begin();BLi!=BoardList.end();BLi++)
     (*BLi)->refreshPieceSet();
@@ -613,6 +675,7 @@ void Global::addHostBookmark(HostBookmark *hbm) {
   }
     
   writeRC();
+  if (bmlistener != 0) bmlistener->updateBookmarks();
 }
 
 void Global::updateScrollBacks() {
@@ -639,6 +702,39 @@ void * Global::safeMalloc(int nbytes) {
   return(malloc(nbytes + (nbytes % 4)));
 }
 
+GdkFont * Global::loadFont(EboardFont i) {
+  GdkFont *f;
+
+  switch(i) {
+
+  case EF_PlayerFont: // player font (default 18 pt)
+    f=gdk_font_load(PlayerFont);
+    if (!f) f=gdk_font_load("-*-helvetica-bold-r-normal--17-*-*-*-*-*-*");
+    if (!f) f=gdk_font_load("-*-helvetica-bold-r-normal--14-*-*-*-*-*-*");
+    if (!f) f=gdk_font_load("helvetica");
+    break;       
+
+  case EF_ClockFont: // clock font (default 34 pt)
+    f=gdk_font_load(ClockFont);
+    if (!f) f=gdk_font_load("-*-helvetica-bold-r-normal--34-*-*-*-*-*-*");
+    if (!f) f=gdk_font_load("-*-helvetica-bold-r-normal--25-*-*-*-*-*-*");
+    if (!f) f=gdk_font_load("-*-helvetica-bold-r-normal--24-*-*-*-*-*-*");
+    if (!f) f=gdk_font_load("helvetica");
+    break;
+
+  case EF_InfoFont: // info font (default 12 pt)
+    f=gdk_font_load(InfoFont);
+    if (!f) f=gdk_font_load("-*-helvetica-medium-r-normal--11-*-*-*-*-*-*");
+    if (!f) f=gdk_font_load("-*-helvetica-medium-r-normal--10-*-*-*-*-*-*");
+    if (!f) f=gdk_font_load("helvetica");
+    break;
+  default:
+    f=NULL;
+  }
+  return f;
+
+}
+
 // ----------
 
 int HostBookmark::operator==(HostBookmark *hbm) {
@@ -857,3 +953,39 @@ void Desktop::spawnConsoles(TextSet *ts) {
     }
   }
 }
+
+// ------- ah, the zombies
+
+ZombieHunter::ZombieHunter() {
+  signal(SIGCHLD,zh_sigchild_handler);
+}
+
+ZombieHunter::~ZombieHunter() {
+  pids.clear();
+  handlers.clear();
+}
+
+void ZombieHunter::add(int pid, SigChildHandler *sigh) {
+  pids.push_back(pid);
+  handlers.push_back(sigh);
+}
+
+void ZombieHunter::handleSigChild() {
+  pid_t epid;
+  int i,s;
+  
+  while ( ( epid = waitpid(-1,&s,WNOHANG) ) > 0 ) {
+    for(i=0;i<pids.size();i++)
+      if (pids[i] == epid) {
+	if (handlers[i] != 0) handlers[i]->ZombieNotification(epid);
+	pids.erase(pids.begin() + i);
+	handlers.erase(handlers.begin() + i);
+	break;
+      }
+  }
+}
+
+void zh_sigchild_handler(int sig) {
+  if (sig == SIGCHLD)
+    global.zombies.handleSigChild();
+}
