@@ -1,10 +1,10 @@
-/* $Id: global.h,v 1.38 2001/09/30 17:48:23 bergo Exp $ */
+/* $Id: global.h,v 1.44 2002/03/17 04:53:36 bergo Exp $ */
 
 /*
 
     eboard - chess client
     http://eboard.sourceforge.net
-    Copyright (C) 2000-2001 Felipe Paulo Guazzi Bergo
+    Copyright (C) 2000-2002 Felipe Paulo Guazzi Bergo
     bergo@seul.org
 
     This program is free software; you can redistribute it and/or modify
@@ -31,6 +31,7 @@
 #include "text.h"
 #include "network.h"
 #include "protocol.h"
+#include "position.h"
 #include "chess.h"
 #include "sound.h"
 #include "seekgraph.h"
@@ -44,8 +45,11 @@ class Status;
 class ProtocolParser;
 class ChessGame;
 class Notebook;
+class QuickBar;
+class QButton;
 class SeekGraph;
 class SoundSlave;
+class StringCollection;
 
 class HostBookmark {
  public:
@@ -55,6 +59,20 @@ class HostBookmark {
   char protocol[64];
 };
 
+class ZombieHunter {
+ public:
+  ZombieHunter();
+  ~ZombieHunter();
+  void add(int pid, SigChildHandler *sigh);
+
+ private:
+  void handleSigChild();
+  friend void zh_sigchild_handler(int sig);
+
+  vector<int> pids;
+  vector<SigChildHandler *> handlers;
+};
+
 class ChannelSplitter {
  public:  
   void appendToChannel(int ch,char *msg,int color,Importance im=IM_NORMAL); 
@@ -135,6 +153,7 @@ class Global : public ChannelSplitter {
 
   void appendGame(ChessGame *cg,bool RenumberDupes=true);
   void prependGame(ChessGame *cg,bool RenumberDupes=true);
+  void deleteGame(ChessGame *cg);
 
   void statOS();
   void ensureDirectories();
@@ -162,16 +181,21 @@ class Global : public ChannelSplitter {
   void opponentMoved();
   void drawOffered();
   void privatelyTold();
+  void challenged();
+
   void flushSound();
   void setPasswordMode(int pm);
   //  void clearSoundStack();
 
   void updateScrollBacks();
+  void dropQuickbarButtons();
 
   bool hasSoundFile(char *p);
 
   Notebook * getNotebook();
 
+  GdkFont *loadFont(EboardFont i);
+
   // free with free() as usual
   static void * safeMalloc(int nbytes);
 
@@ -194,12 +218,19 @@ class Global : public ChannelSplitter {
   Notebook           *ebook;
   SeekGraph          *skgraph;
   History            *inputhistory;
+  BookmarkListener   *bmlistener;
+  UpdateInterface    *qbcontainer;
+  QuickBar           *quickbar;
 
+  StringCollection   annotator;
   SoundSlave         sndslave;
+  ZombieHunter       zombies;
 
   int                SelfInputColor;
   int                PasswordMode;
 
+  int                LastScratch;
+
   int HilightLastMove;
   int AnimateMoves;
   int Premove;
@@ -238,12 +269,14 @@ class Global : public ChannelSplitter {
   int SmartDiscard;
 
   int ShowCoordinates;
+  int ShowQuickbar;
+  bool PopupHelp;
 
   TerminalColor Colors;
   Desktop Desk;
   
-  // 0=opponent moved, 1=draw offer 2=pvt tell
-  SoundEvent sndevents[3];
+  // 0=opponent moved, 1=draw offer 2=pvt tell 3=challenged
+  SoundEvent sndevents[4];
 
   int CommLog;
   int DebugLog;
@@ -254,6 +287,7 @@ class Global : public ChannelSplitter {
   list<HostBookmark *> HostHistory;
 
   vector<string> SoundFiles;
+  vector<QButton *> QuickbarButtons;
 
   char * Version;
   char   SystemType[64];
