@@ -1,4 +1,4 @@
-/* $Id: help.h,v 1.1 2001/04/28 17:07:01 bergo Exp $ */
+/* $Id: help.h,v 1.4 2002/02/13 03:53:56 bergo Exp $ */
 
 /*
 
@@ -27,6 +27,7 @@
 #define EBOARD_HELP_H 1
 
 #include "widgetproxy.h"
+#include "notebook.h"
 
 namespace Help {
 
@@ -35,6 +36,25 @@ namespace Help {
     KeysDialog();
   };
 
+  class DebugDialog : public ModalDialog {
+  public:
+    DebugDialog();
+
+  private:
+    char *grabOutput(char *cmdline);
+    char pipedata[1024];
+
+    friend void debugdlg_writecons(GtkWidget *w, gpointer data);
+  };
+
+  class GettingStarted : public NonModalDialog {
+  public:
+    GettingStarted();
+  private:
+    void compose();
+    GtkWidget *text;
+  };
 }
 
+
 #endif
