@@ -1,10 +1,10 @@
-/* $Id: mainwindow.cc,v 1.72 2001/10/01 01:56:04 bergo Exp $ */
+/* $Id: mainwindow.cc,v 1.81 2002/03/17 04:53:36 bergo Exp $ */
 
 /*
 
     eboard - chess client
     http://eboard.sourceforge.net
-    Copyright (C) 2000-2001 Felipe Paulo Guazzi Bergo
+    Copyright (C) 2000-2002 Felipe Paulo Guazzi Bergo
     bergo@seul.org
 
     This program is free software; you can redistribute it and/or modify
@@ -47,6 +47,7 @@
 #include "movelist.xpm"
 #include "flip.xpm"
 #include "trash.xpm"
+#include "toscratch.xpm"
 
 #include "sealoff.xpm"
 #include "sealon.xpm"
@@ -56,6 +57,7 @@ GdkWindow * MainWindow::RefWindow=0;
 static GtkItemFactoryEntry mainwindow_mainmenu[] = {
   { "/_Peer",                NULL,         NULL, 0, "<Branch>" },
   { "/Peer/Connect to _FICS",NULL,GTK_SIGNAL_FUNC(peer_connect_fics),0,NULL },
+  { "/Peer/ICS _Bookmarks",NULL, NULL, 0, "<Branch>" },
   { "/Peer/_Connect to Other Server...",NULL,GTK_SIGNAL_FUNC(peer_connect_ask),0,NULL },
   { "/Peer/sep4",            NULL,           NULL, 0, "<Separator>" },
   { "/Peer/Play against _Engine",NULL, NULL, 0,"<Branch>" },
@@ -65,6 +67,9 @@ static GtkItemFactoryEntry mainwindow_mainmenu[] = {
   { "/Peer/Play against Engine/sep1",NULL,NULL,0,"<Separator>" },
   { "/Peer/Play against Engine/_Generic Engine...",NULL,GTK_SIGNAL_FUNC(peer_connect_xboard), 0,NULL },
   { "/Peer/sep3",            NULL,           NULL, 0, "<Separator>" },
+  { "/Peer/_Empty Scratch Board",     NULL,  GTK_SIGNAL_FUNC(peer_scratch_empty), 0, NULL },
+  { "/Peer/_Scratch Board with Initial Position",NULL,  GTK_SIGNAL_FUNC(peer_scratch_initial), 0, NULL },
+  { "/Peer/sep3",            NULL,           NULL, 0, "<Separator>" },
   { "/Peer/_Disconnect",     NULL,  GTK_SIGNAL_FUNC(peer_disconnect), 0, NULL },
   { "/Peer/sep2",            NULL,           NULL, 0, "<Separator>" },
   { "/Peer/_Quit",           NULL,  GTK_SIGNAL_FUNC(mainwindow_destroy), 0, NULL },
@@ -111,7 +116,9 @@ static GtkItemFactoryEntry mainwindow_mainmenu[] = {
   { "/Windows/sep3", NULL, NULL, 0, "<Separator>" },
   { "/Windows/Save Desktop _Geometry", NULL, GTK_SIGNAL_FUNC(windows_savedesk), 0, NULL },
   { "/_Help",              NULL,         NULL,0, "<Branch>" },
+  { "/Help/_Getting Started",NULL, GTK_SIGNAL_FUNC(help_starting), 0, NULL },
   { "/Help/_Keys",NULL, GTK_SIGNAL_FUNC(help_keys), 0, NULL },
+  { "/Help/_Debug Info",NULL, GTK_SIGNAL_FUNC(help_debug), 0, NULL },
   { "/Help/sep4",            NULL,           NULL, 0, "<Separator>" },
   { "/Help/_About eboard...",NULL,  GTK_SIGNAL_FUNC(help_about), 0, NULL },
 
@@ -132,8 +139,9 @@ MainWindow::MainWindow() {
   stocklist=0;
   adlist=0;
   consolecopy=0;
+  scriptlist=0;
 
-  for(i=0;i<7;i++)
+  for(i=0;i<8;i++)
     nav_enable[i]=true;
 
   widget=gtk_window_new(GTK_WINDOW_TOPLEVEL);
@@ -161,6 +169,9 @@ MainWindow::MainWindow() {
   searchThemes();
   global.readRC();
 
+  /* build bookmark submenu under Peer */
+  updateBookmarks();
+
   /* promotion menu */
   promote=new PromotionPicker(widget->window);
 
@@ -188,6 +199,14 @@ MainWindow::MainWindow() {
   notebook->addPage(tmp->widget,"Main Board",-1);
   tmp->setNotebook(notebook,-1);
 
+  /* quick bar */
+  quickbar=new QuickBar(widget);
+  gtk_box_pack_start(GTK_BOX(v), quickbar->widget, FALSE,TRUE, 0);
+  quickbar->show();
+  QuickbarVisible=true;
+  if (!global.ShowQuickbar)
+    hideQuickbar();
+
   icsout=new TextSet();
   inconsole=new Text();
   icsout->addTarget(inconsole);
@@ -298,6 +317,9 @@ MainWindow::MainWindow() {
   global.promotion=(PieceProvider *)promote;
   global.ebook=notebook;
   global.inputhistory=InputHistory;
+  global.bmlistener=(BookmarkListener *)this;
+  global.qbcontainer=(UpdateInterface *)this;
+  global.quickbar=quickbar;
   mainw=this;
 
   notebook->setListener(this);
@@ -309,6 +331,28 @@ MainWindow::MainWindow() {
     gtk_window_set_default_size(GTK_WINDOW(widget),-1,-1);
 }
 
+void MainWindow::update() {
+  bool dState;
+  dState = global.ShowQuickbar ? true : false;
+  if (dState != QuickbarVisible) {
+    if (QuickbarVisible) hideQuickbar(); else showQuickbar();
+  }
+}
+
+void MainWindow::showQuickbar() {
+  if (!QuickbarVisible) {
+    quickbar->show();
+    QuickbarVisible = true;
+  }
+}
+
+void MainWindow::hideQuickbar() {
+  if (QuickbarVisible) {
+    quickbar->hide();
+    QuickbarVisible = false;
+  }
+}
+
 void MainWindow::setSealPix(int flag) {
   gtk_pixmap_set(GTK_PIXMAP(picseal),sealmap[flag?1:0],
 		 sealmask[flag?1:0]);
@@ -339,9 +383,9 @@ void MainWindow::createSealPix(GtkWidget *box) {
 }
 
 void MainWindow::createNavbar(GtkWidget *box) {
-  GdkPixmap *d[7];
-  GdkBitmap *mask[7];
-  GtkWidget *p[7],*b[7],*fr,*mb,*lb;
+  GdkPixmap *d[8];
+  GdkBitmap *mask[8];
+  GtkWidget *p[8],*b[8],*fr,*mb,*lb;
   GtkStyle *style;
   int i;
 
@@ -367,6 +411,9 @@ void MainWindow::createNavbar(GtkWidget *box) {
   d[6] = gdk_pixmap_create_from_xpm_d (widget->window, &mask[6],
                                        &style->bg[GTK_STATE_NORMAL],
                                        (gchar **) trash_xpm);  
+  d[7] = gdk_pixmap_create_from_xpm_d (widget->window, &mask[7],
+                                       &style->bg[GTK_STATE_NORMAL],
+                                       (gchar **) toscratch_xpm);  
   mb=gtk_hbox_new(FALSE,0);
   fr=gtk_frame_new(0);
   gtk_frame_set_shadow_type(GTK_FRAME(fr),GTK_SHADOW_ETCHED_OUT);
@@ -375,7 +422,7 @@ void MainWindow::createNavbar(GtkWidget *box) {
   lb=gtk_label_new("Game/Board: ");
   gtk_box_pack_start(GTK_BOX(mb),lb,FALSE,FALSE,2);
 
-  for(i=0;i<7;i++) {
+  for(i=0;i<8;i++) {
     b[i]=gtk_button_new();
     p[i]=gtk_pixmap_new(d[i],mask[i]);
     gtk_container_add(GTK_CONTAINER(b[i]),p[i]);
@@ -398,6 +445,7 @@ void MainWindow::createNavbar(GtkWidget *box) {
   gtk_tooltips_set_tip(tooltips,b[4],"pops up the move list",0);
   gtk_tooltips_set_tip(tooltips,b[5],"flips board",0);
   gtk_tooltips_set_tip(tooltips,b[6],"discards board",0);
+  gtk_tooltips_set_tip(tooltips,b[7],"opens new scratch board with position",0);
 
   gtk_signal_connect(GTK_OBJECT(b[0]),"clicked",
 		     GTK_SIGNAL_FUNC(navbar_back_all),this);
@@ -413,6 +461,8 @@ void MainWindow::createNavbar(GtkWidget *box) {
 		     GTK_SIGNAL_FUNC(navbar_flip),this);
   gtk_signal_connect(GTK_OBJECT(b[6]),"clicked",
 		     GTK_SIGNAL_FUNC(navbar_trash),this);
+  gtk_signal_connect(GTK_OBJECT(b[7]),"clicked",
+		     GTK_SIGNAL_FUNC(navbar_toscratch),this);
 }
 
 void MainWindow::setTitle(char *msg) {
@@ -492,6 +542,40 @@ void MainWindow::parseThemeFile(char *name) {
   fclose(f);
 }
 
+void MainWindow::updateBookmarks() {
+  GtkWidget *pmenu, *item;
+  GList *r,*s;
+  list<HostBookmark *>::iterator hi;
+  char z[256];
+
+  pmenu=gtk_item_factory_get_widget(gif,"/Peer/ICS Bookmarks");
+
+  r=gtk_container_children(GTK_CONTAINER(pmenu));
+  for(s=r;s!=0;s=g_list_next(s))
+    gtk_container_remove( GTK_CONTAINER(pmenu), GTK_WIDGET(s->data) );
+  g_list_free(r);
+
+  for(hi=global.HostHistory.begin();hi!=global.HostHistory.end();hi++) {
+    snprintf(z,256,"Connect to %s:%d (%s)",(*hi)->host,(*hi)->port,(*hi)->protocol);
+    z[255]=0;
+    item=gtk_menu_item_new_with_label( z );
+
+    gtk_signal_connect(GTK_OBJECT(item),"activate",
+		       GtkSignalFunc(mainwindow_connect_bookmark),
+		       (gpointer)(*hi));
+
+    gtk_menu_append(GTK_MENU(pmenu),item);
+    gtk_widget_show(item);    
+  }
+
+  if (global.HostHistory.empty()) {
+    item=gtk_menu_item_new_with_label("(no bookmarks)");
+    gtk_widget_set_sensitive(item,FALSE);
+    gtk_menu_append(GTK_MENU(pmenu),item);
+    gtk_widget_show(item);    
+  }
+}
+
 void MainWindow::searchThemes() {
   EboardFileFinder eff;
   list<ThemeEntry *>::iterator it;
@@ -571,7 +655,7 @@ void MainWindow::greet() {
   char z[128];
   sprintf (z,"eboard version %s (%s)",global.Version,global.SystemType);
   icsout->append(z,0xc0ff00,IM_IGNORE);
-  icsout->append("(c) 2001 Felipe Bergo <bergo@seul.org> (FICS Handle: Pulga)",0xc0ff00,IM_IGNORE);
+  icsout->append("(c) 2001-2002 Felipe Bergo <bergo@seul.org> (FICS Handle: Pulga)",0xc0ff00,IM_IGNORE);
   icsout->append("Distributed under the terms of the GNU General Public License, version 2 or later",0xffc000,IM_IGNORE);
   icsout->append("http://www.gnu.org/copyleft/gpl.html",0xffc000,IM_IGNORE);
   icsout->append("Source code available at http://eboard.sourceforge.net",0xc0ff00,IM_IGNORE);
@@ -792,8 +876,6 @@ void MainWindow::tryConnect(char *host,int port,ProtocolParser *protocol,
 
 void MainWindow::disconnect() {
   global.debug("MainWindow","disconnect");
-  if ((global.network)&&(global.protocol))
-    global.protocol->finalize();
   cleanUpConnection();
 }
 
@@ -808,6 +890,7 @@ void MainWindow::cleanUpConnection() {
     icsout->append("--- Disconnected",0xc0ff00);
   }
   if (global.protocol) {
+    global.protocol->finalize();
     delete global.protocol;
     global.protocol=0;
   }
@@ -819,6 +902,62 @@ void MainWindow::cleanUpConnection() {
   status->setText("No peer.");
 }
 
+void MainWindow::cloneOnScratch(ChessGame *cg0) {
+  Board *b;
+  ChessGame *cg;
+  Position *p;
+  int id,i,j;
+  char z[64];
+
+  id = global.nextFreeGameId(30000);
+  cg=new ChessGame(cg0);
+  cg->GameNumber = id;
+  cg->LocalEdit = true;
+
+  global.appendGame(cg,false);
+  b=new EditBoard(cg);
+  cg->setBoard(b);
+  cg->StopClock = 1;
+
+  sprintf(z,"Scratch %d",++global.LastScratch);
+  notebook->addPage(b->widget,z,id);
+  b->setNotebook(notebook,id);
+  notebook->goToPageId(id);
+  b->update();
+}
+
+void MainWindow::newScratchBoard(bool clearboard) {
+  Board *b;
+  ChessGame *cg;
+  Position *p;
+  int id,i,j;
+  char z[64];
+
+  id = global.nextFreeGameId(30000);
+
+  cg=new ChessGame(id, 0, 0, 0, REGULAR,
+		   "Nobody","Nessuno");
+  cg->LocalEdit=true;
+
+  global.appendGame(cg,false);
+  b=new EditBoard(cg);
+  cg->setBoard(b);
+  cg->StopClock = 1;
+
+  sprintf(z,"Scratch %d",++global.LastScratch);
+  notebook->addPage(b->widget,z,id);
+  b->setNotebook(notebook,id);
+
+  p=new Position();
+  if (clearboard)
+    for(i=0;i<8;i++)
+      for(j=0;j<8;j++)
+	p->setPiece(i,j,EMPTY);
+
+  cg->updatePosition(*p,1,0,0,0,"<editing>",false);
+  notebook->goToPageId(id);
+}
+
 void MainWindow::gameWalk(int op) {
   Board *b=0;
   ChessGame *cg=0;
@@ -861,12 +1000,14 @@ void MainWindow::gameWalk(int op) {
   case 4: b->openMovelist(); break;
   case 5:
     if ((id<=0)||(!cg)) break;
-    if ( (cg->protodata[1]) || ( ! global.protocol ) ) {
+    if ( (cg->protodata[1]) || ( ! global.protocol ) || (cg->LocalEdit) ) {
       cg->protodata[1]=0;
       global.ebook->removePage(cg->GameNumber);
       global.removeBoard(b);
       delete(b);
       cg->setBoard(0);
+      if (cg->LocalEdit)
+	global.deleteGame(cg);
     } else {
       if ((id>0)&&(global.protocol))
 	global.protocol->discardGame(cg->GameNumber);
@@ -875,6 +1016,9 @@ void MainWindow::gameWalk(int op) {
   case 6:
     b->setFlipInversion(!b->getFlipInversion());
     break;
+  case 7: // clone on scratch
+    cloneOnScratch(b->getGame());
+    break;
   }
 }
 
@@ -904,28 +1048,29 @@ gboolean do_smart_remove(gpointer data) {
 // changes enabling of buttons upon pane change
 // also schedules 'smart removes' for finished boards
 void MainWindow::paneChanged(int pgseq,int pgid) {
-  bool nv[7];
+  bool nv[8];
   int i;
 
   // #     button(s)  constraint for enabling
   // ------------------------------------------------------
-  // 0..3  game move  requires game
-  // 4     move list  requires game
-  // 5     flip       requires board
-  // 6     trash      requires board other than main board
+  // 0..3  game move   requires game
+  // 4     move list   requires game
+  // 5     flip        requires board
+  // 6     trash       requires board other than main board
+  // 7     new scratch requires game
 
   // main board: -1  console: -2  seek graph: -3
   // channel tabs: -200 .. -455 (for FICS, formula is -(200+channelnum) )
   // other boards: >= 0
   // most probably computer thinking pane will be -4
 
-  for(i=0;i<7;i++) nv[i]=true;
+  for(i=0;i<8;i++) nv[i]=true;
 
   // 0..4
   if (pgid < -1) nv[0]=false;
   if ( (pgid == -1) && (! global.BoardList.front()->hasGame()) ) nv[0]=false;
 
-  nv[1]=nv[2]=nv[3]=nv[4]=nv[0];
+  nv[1]=nv[2]=nv[3]=nv[4]=nv[7]=nv[0];
 
   // 5
   if (pgid < -1) nv[5]=false;
@@ -933,7 +1078,7 @@ void MainWindow::paneChanged(int pgseq,int pgid) {
   // 6
   if (pgid < 0) nv[6]=false;
 
-  for(i=0;i<7;i++) {
+  for(i=0;i<8;i++) {
     if (nv[i]!=nav_enable[i]) {
       gtk_widget_set_sensitive(navbar[i],nv[i]?TRUE:FALSE);
       nav_enable[i]=nv[i];
@@ -1100,6 +1245,16 @@ peer_disconnect(gpointer data) {
   mainw->disconnect();
 }
 
+void
+peer_scratch_empty(gpointer data) {
+  mainw->newScratchBoard(true);
+}
+
+void
+peer_scratch_initial(gpointer data) {
+  mainw->newScratchBoard(false);
+}
+
 void
 peer_connect_fics(gpointer data) {
   mainw->openServer("freechess.org",5000,new FicsProtocolParser(),"timeseal");
@@ -1189,6 +1344,16 @@ help_keys(gpointer data) {
   (new Help::KeysDialog())->show();
 }
 
+void
+help_debug(gpointer data) {
+  (new Help::DebugDialog())->show();
+}
+
+void
+help_starting(gpointer data) {
+  (new Help::GettingStarted())->show();
+}
+
 gboolean
 mainwindow_idle_hook(gpointer data) {
   MainWindow *me;
@@ -1297,6 +1462,13 @@ navbar_trash(GtkWidget *w,gpointer data) {
   me->gameWalk(5);
 }
 
+void
+navbar_toscratch(GtkWidget *w,gpointer data) {
+  MainWindow *me;
+  me=(MainWindow *)data;
+  me->gameWalk(7);
+}
+
 void
 navbar_flip(GtkWidget *w,gpointer data) {
   MainWindow *me;
@@ -1410,6 +1582,18 @@ mainwindow_gcp_about_to_get_angry(GtkWidget *widget,GdkEventFocus *event,gpointe
   return TRUE;
 }
 
+void
+mainwindow_connect_bookmark(GtkWidget *w, gpointer data) {
+  HostBookmark *bm;  
+  char p[32];
+  bm=(HostBookmark *)data;
+
+  // gcc 2.95 and up are just annoying with this const / non-const thing
+  strcpy(p,"timeseal");
+  global.chandler->openServer(bm->host,bm->port,new FicsProtocolParser(),
+			      strcmp(bm->protocol,"FICS") ? 0 : p);
+}
+
 // ThemeEntry
 
 bool ThemeEntry::isDupe(ThemeEntry *te) {
