@@ -1,10 +1,10 @@
-/* $Id: network.cc,v 1.24 2001/10/24 11:10:22 bergo Exp $ */
+/* $Id: network.cc,v 1.28 2002/03/12 02:03:23 bergo Exp $ */
 
 /*
 
     eboard - chess client
     http://eboard.sourceforge.net
-    Copyright (C) 2000-2001 Felipe Paulo Guazzi Bergo
+    Copyright (C) 2000-2002 Felipe Paulo Guazzi Bergo
     bergo@seul.org
 
     This program is free software; you can redistribute it and/or modify
@@ -75,12 +75,11 @@ void PidRing::funeral(int pid) {
 
 PidRing netring;
 
-// handles SIGCHLD
-static void dead_kid(int signum) {
-  pid_t dpid;
-  dpid=wait(0);
-  if (dpid>0)
-    netring.funeral(dpid);
+static pid_t last_dead_pid = 0;
+
+void PidRing::ZombieNotification(int pid) {
+  funeral(pid);
+  last_dead_pid = pid; // for use in PipeConnection::open()
 }
 
 // ===================================================================
@@ -503,7 +502,6 @@ int PipeConnection::open() {
     strcpy(errorMessage,"IPC pipe creation failed.");
     return -1;    
   }
-  signal(SIGCHLD,dead_kid); // if not already
   signal(SIGPIPE,SIG_IGN);
 
   pid=fork();
@@ -527,11 +525,23 @@ int PipeConnection::open() {
     ::close(h2n[1]);
     memset(firstline,0,256);
     p=firstline;
-    while(1)
+    while(1) {
       if (read(h2n[0],p,1)==1) {
 	if (*p=='\n') break;
 	++p;
       }
+
+      // should fix splange's crashing bug
+      if ( (last_dead_pid == pid) || 
+           (waitpid(pid, 0, WNOHANG) != 0) ) {
+	last_dead_pid = 0;
+	strcpy(errorMessage,"Program exited too soon");
+	::close(h2n[0]);
+	::close(n2h[1]);
+	return -1;
+      }
+
+    }
     *p=0;
     if (!strcmp(firstline,"exec failed")) {
       strcpy(errorMessage,"Failed to run helper program");
@@ -546,6 +556,7 @@ int PipeConnection::open() {
   fcntl(pin,F_SETFL,O_NONBLOCK);
   global.TheOffspring.push_back(-pid);
   netring.add(this,pid);
+  global.zombies.add(pid,&netring);
 
   switch(opmode) {
   case 0:
