@@ -1,10 +1,10 @@
-/* $Id: position.cc,v 1.32 2001/07/31 04:14:17 bergo Exp $ */
+/* $Id: position.cc,v 1.35 2002/02/13 03:53:56 bergo Exp $ */
 
 /*
 
     eboard - chess client
     http://eboard.sourceforge.net
-    Copyright (C) 2000-2001 Felipe Paulo Guazzi Bergo
+    Copyright (C) 2000-2002 Felipe Paulo Guazzi Bergo
     bergo@seul.org
 
     This program is free software; you can redistribute it and/or modify
@@ -33,6 +33,78 @@
 #include "position.h"
 #include "global.h"
 
+StringCollection::StringCollection() {
+  amOpen = false;
+  LastId = 0;
+}
+
+StringCollection::~StringCollection() {
+  collection.clear();
+}
+
+int StringCollection::open() {
+  IntAndString s;
+  if (amOpen) return LastId;
+  s.i = ++LastId;
+  collection.push_back(s);
+  amOpen=true;
+  return LastId;
+}
+
+void StringCollection::append(char *s) {
+  if (amOpen)
+    collection.back().s.append(s);
+}
+
+void StringCollection::append(char c) {
+  if (amOpen)
+    collection.back().s.append(1,c);
+}
+
+void StringCollection::close() {
+  if (amOpen)
+    amOpen=false;
+}
+
+bool StringCollection::isOpen() {
+  return(amOpen);
+}
+
+const char * StringCollection::get(int id) {
+  list<IntAndString>::iterator li;
+
+  for(li=collection.begin();li!=collection.end();li++)
+    if ( (*li).i == id )
+      return ((*li).s.c_str());
+
+  return 0;
+}
+
+void StringCollection::link(int id) {
+  list<IntAndString>::iterator li;
+  for(li=collection.begin();li!=collection.end();li++)
+    if ( (*li).i == id ) {
+      (*li).references++;
+      // cout << "link id " << id << " references " << (*li).references << endl;
+      break;
+    }
+}
+
+void StringCollection::unlink(int id) {
+  list<IntAndString>::iterator li;
+  for(li=collection.begin();li!=collection.end();li++)
+    if ( (*li).i == id ) {
+      (*li).references--;
+      /*
+      if ( ! (*li).references )
+	collection.erase(li);
+      */
+      break;
+    }
+}
+
+// -------------------------------------------------
+
 SMove::SMove(piece p,int sx,int sy,int dx,int dy) {
   Piece=p;
   SX=sx; SY=sy;
@@ -47,11 +119,24 @@ int SMove::distance() {
   return((int)hypot((double)(DX-SX),(double)(DY-SY)));
 }
 
+char Position::AnnotationBuffer[512];
+
 Position::Position() {
   setStartPos();
   LastMove[0]=0;
 }
 
+Position::~Position() {
+  dropAnnotations();
+}
+
+void Position::dropAnnotations() {
+  list<int>::iterator ai;
+  for(ai=Annotations.begin();ai!=Annotations.end();ai++)
+    global.annotator.unlink(*ai);
+  Annotations.clear();
+}
+
 void Position::setStartPos() {
   register int i,j;
   for(i=0;i<8;i++)
@@ -94,6 +179,8 @@ int Position::operator!=(Position &p) {
 
 Position Position::operator=(Position &p) {
   int i,j;
+  list<int>::iterator ai;
+
   for(i=0;i<8;i++)
     for(j=0;j<8;j++)
       square[i][j]=p.square[i][j];
@@ -105,6 +192,13 @@ Position Position::operator=(Position &p) {
   ep[0]=p.ep[0];
   ep[1]=p.ep[1];
   strcpy(LastMove,p.LastMove);
+
+  dropAnnotations();
+  for(ai=p.Annotations.begin();ai!=p.Annotations.end();ai++) {
+    Annotations.push_back(*ai);
+    global.annotator.link(*ai);
+  }
+
   return(*this);
 }
 
@@ -116,6 +210,28 @@ piece Position::getPiece(int col,int row) {
   return(square[col%8][row%8]);
 }
 
+void Position::addAnnotation(int id) {
+  Annotations.push_back(id);
+  global.annotator.link(id);
+}
+
+char * Position::getAnnotation() {
+  list<int>::iterator ai;
+
+  if (Annotations.empty())
+    return 0;
+
+  AnnotationBuffer[0]=0;
+  for(ai=Annotations.begin();ai!=Annotations.end();ai++)
+    strncat(AnnotationBuffer, global.annotator.get(*ai), 511);
+
+  return(AnnotationBuffer);
+}
+
+void Position::clearAnnotation() {
+  Annotations.clear();
+}
+
 void Position::invalidate() {
   piece c;
   int i,j;
@@ -134,7 +250,7 @@ char * Position::getLastMove() {
 }
 
 void Position::moveAnyNotation(char *m,piece color,variant Vr=REGULAR) {
-  int i,x,y;
+  int i,x,y,ml;
   piece pr;
   char xlate[12];
   global.debug("Position","moveAnyNotation",m);
@@ -158,31 +274,36 @@ void Position::moveAnyNotation(char *m,piece color,variant Vr=REGULAR) {
     moveDrop(pr|color,x,y);
     return;
   }
+  
+  ml = strlen(m);
 
-  if ( (isupper(m[0])) || (m[1]=='x') || (m[2]=='x') || (m[0]=='o') || (strlen(m)==2) || (m[2]=='=') || ((strlen(m)>=3)&&((m[2]=='+')||(m[2]=='#'))) ) {
-    if (m[0]=='o') { // cope with o-o for castling (gnu chess 4)
-      strcpy(xlate,m);
-      for(i=0;xlate[i];i++)
-	xlate[i]=toupper(xlate[i]);
-      moveStdNotation(xlate,color,Vr);
-    } else      
-      moveStdNotation(m,color,Vr);
-  } else {
-    if (strlen(m)<4)
-      return;
-    moveCartesian(m[0]-'a',m[1]-'1',x=(m[2]-'a'),y=(m[3]-'1'),Vr);
-    if (m[4]!=0) { // promotion
-      switch(m[4]) {
-      case 'q': pr=QUEEN; break;
-      case 'r': pr=ROOK; break;
-      case 'n': pr=KNIGHT; break;
-      case 'b': pr=BISHOP; break;
-      case 'k': pr=KING; break;
-      default: return;
+  if ( isupper(m[0]) || m[1]=='x' || m[2]=='x' || m[0]=='o' || ml==2 || 
+       ml == 3 || m[2]=='=' || ( ml >= 4 && ( m[3]=='+' || m[3]=='#' ) ) 
+       ) 
+    {
+      if (m[0]=='o') { // cope with o-o for castling (gnu chess 4)
+	strcpy(xlate,m);
+	for(i=0;xlate[i];i++)
+	  xlate[i]=toupper(xlate[i]);
+	moveStdNotation(xlate,color,Vr);
+      } else      
+	moveStdNotation(m,color,Vr);
+    } else {
+      if (ml < 4)
+	return;
+      moveCartesian(m[0]-'a',m[1]-'1',x=(m[2]-'a'),y=(m[3]-'1'),Vr);
+      if (m[4]!=0) { // promotion
+	switch(m[4]) {
+	case 'q': pr=QUEEN; break;
+	case 'r': pr=ROOK; break;
+	case 'n': pr=KNIGHT; break;
+	case 'b': pr=BISHOP; break;
+	case 'k': pr=KING; break;
+	default: return;
+	}
+	square[x][y]=pr|color|WASPAWN;
       }
-      square[x][y]=pr|color|WASPAWN;
     }
-  }
 }
 
 void Position::moveStdNotation(char *m,piece color,variant Vr=REGULAR) {
@@ -242,8 +363,11 @@ void Position::moveStdNotation(char *m,piece color,variant Vr=REGULAR) {
       to[1]=*p-'1';
       continue;
     }
-    if (*p == '=') {
-      ++p;
+
+    // promotion. cope also with bad notation in PGNs (a8Q instead of a8=Q)
+    
+    if ( what==PAWN && to[1]%7 == 0 && strchr("=QqRrBbNnKk",*p) ) {
+      if (*p == '=') ++p;
       switch(*p) {
       case 'Q':
       case 'q': prom=QUEEN; break;
@@ -258,7 +382,7 @@ void Position::moveStdNotation(char *m,piece color,variant Vr=REGULAR) {
       }
       continue;
     }
-  }
+  } // for
 
   if ((isdrop)&&(getStockCount(what|color))&&
       (to[0]>=0)&&(to[1]>=0)&&(square[to[0]][to[1]]==EMPTY)) {
@@ -276,6 +400,9 @@ void Position::moveStdNotation(char *m,piece color,variant Vr=REGULAR) {
     if ((Vr==BUGHOUSE)||(Vr==CRAZYHOUSE))
       if (square[to[0]][to[1]]!=EMPTY)
 	incStockCount( square[to[0]][to[1]]^COLOR_MASK );
+      else 
+	if (what==PAWN)
+	  incStockCount( square[from[0]][from[1]] ); // en passant
 
   // en passant takes
   if ((istake)&&(square[to[0]][to[1]]==EMPTY)&&(what==PAWN)) {
