@@ -1,10 +1,10 @@
-/* $Id: proto_fics.cc,v 1.62 2001/10/13 00:22:10 bergo Exp $ */
+/* $Id: proto_fics.cc,v 1.67 2002/03/17 20:51:47 bergo Exp $ */
 
 /*
 
     eboard - chess client
     http://eboard.sourceforge.net
-    Copyright (C) 2000-2001 Felipe Paulo Guazzi Bergo
+    Copyright (C) 2000-2002 Felipe Paulo Guazzi Bergo
     bergo@seul.org
 
     This program is free software; you can redistribute it and/or modify
@@ -50,6 +50,7 @@ FicsProtocolParser::FicsProtocolParser() {
   PActions.push_back(new string("Log"));
 
   AuthenticationGone=0;
+  LectureBotSpecial=false;
   LastWasPrompt=0;
   LastWasCTell=-1;
 
@@ -153,6 +154,13 @@ FicsProtocolParser::FicsProtocolParser() {
   // bughouse stuff
   PTell.set("%a (your partner) tells you: *");
 
+  // challenge
+  Challenge.set("Challenge: *");
+
+  // FICS LectureBot fake channel tells
+  LectureBotXTell.set(":LectureBot(TD)(67):*");
+  LectureBotXTellContinued.set(":   *");
+
   memset(style12table,0,256*sizeof(piece));
   style12table['-']=EMPTY;
   style12table['P']=PAWN|WHITE;
@@ -179,7 +187,8 @@ FicsProtocolParser::FicsProtocolParser() {
 	     &MovesEnd,0);
   Binder.add(&Kibitz,&Whisper,&Say,&Shout,&cShout,&It,&ChannelTell,
 	     &Seek,&Notify,&SgClear,&SgAdd,&SgAdd2,&SgRemove,
-	     &PTell,&Say2,0);
+	     &PTell,&Say2,&Challenge,&LectureBotXTell,
+	     &LectureBotXTellContinued,0);
 }
 
 FicsProtocolParser::~FicsProtocolParser() {
@@ -189,7 +198,8 @@ FicsProtocolParser::~FicsProtocolParser() {
 
 void FicsProtocolParser::finalize() {
   global.debug("FicsProtocolParser","finalize");
-  global.network->writeLine("quit");  
+  if (global.network)
+    global.network->writeLine("quit");  
 }
 
 void FicsProtocolParser::receiveString(char *netstring) {
@@ -437,9 +447,13 @@ void FicsProtocolParser::receiveString(char *netstring) {
     else
       LastWasCTell=-1;
 
+    if (Challenge.match()) {
+      global.challenged();
+    }
+
     if (UserName.match())    {
       msgcolor=global.Colors.PrivateTell; ispersonal=1;
-      if (ChannelTell.match())
+      if (ChannelTell.match() || LectureBotSpecial || LectureBotXTell.match() )
 	goto it_is_a_chtell_too;
       else
 	goto just_output;
@@ -466,8 +480,6 @@ void FicsProtocolParser::receiveString(char *netstring) {
       msgcolor=global.Colors.NewsNotify; ispersonal=0;
     }
 
-    if (netstring[0]==':')  msgcolor= global.Colors.Mamer; // mamer messages
-
     if (Kibitz.match()||Whisper.match()) {
       msgcolor= global.Colors.KibitzWhisper;
       goto just_output;
@@ -479,11 +491,47 @@ void FicsProtocolParser::receiveString(char *netstring) {
     }
 
     if (Seek.match()) {
-      msgcolor= global.Colors.Seeks;;
+      msgcolor= global.Colors.Seeks;
       goto just_output;
     }
 
  it_is_a_chtell_too:
+
+    if (netstring[0]==':')  {
+      if (!ispersonal)
+	msgcolor= global.Colors.Mamer; // mamer messages
+      
+      if (global.SplitChannels) {
+
+	// I am just pasting the continuation of a LectureBot fake chtell
+	if (LectureBotSpecial) {
+
+	  if (LectureBotXTellContinued.match()) {
+	    global.appendToChannel(67,netstring,msgcolor,
+				   (ispersonal)?IM_PERSONAL:IM_NORMAL);
+	    if (!global.ChannelsToConsoleToo)
+	      goto dont_even_output;
+	  } else
+	    LectureBotSpecial = false;
+
+	} else {
+
+	  // LectureBot is starting a fake chtell
+	  if (LectureBotXTell.match()) {
+	    global.appendToChannel(67,netstring,msgcolor,
+				   (ispersonal)?IM_PERSONAL:IM_NORMAL);
+	    LectureBotSpecial=true;
+	    if (!global.ChannelsToConsoleToo)
+	      goto dont_even_output;
+	  }
+
+
+	}
+      } else // this would never happen without split channels
+	LectureBotSpecial=false;
+
+    } // mamer et al.
+
     if ((global.SplitChannels)&&(LastWasCTell>=0)&&(netstring[0]=='\\')) {      
       global.appendToChannel(LastWasCTell,netstring,msgcolor,
 			     (ispersonal)?IM_PERSONAL:IM_NORMAL);
@@ -492,7 +540,8 @@ void FicsProtocolParser::receiveString(char *netstring) {
     }
 
     if (ChannelTell.match()) {
-      msgcolor = global.Colors.ChannelTell;
+      if (!ispersonal)
+	msgcolor = global.Colors.ChannelTell;
 
       if (global.SplitChannels) {
 	int ch;
@@ -865,6 +914,7 @@ void FicsProtocolParser::parseStyle12(char *data) {
   char stra[64],strb[64],strc[64];
   char plyr[2][64];
   int ackp=0;
+  bool spawnExamination;
 
   global.debug("FicsProtocolParser","parseStyle12");
 
@@ -912,10 +962,19 @@ void FicsProtocolParser::parseStyle12(char *data) {
 
   switch(rel) {
   case 2: // I am the examiner of this game
+    spawnExamination=false;
     refgame=global.getGame(game);
+
     if (!refgame)
+      spawnExamination=true;      
+    else // v_examine=1 on FICS: played game still on main board but inactive
+      if ( (refgame->protodata[0]!=258) && (refgame->isOver()) )
+	spawnExamination=true;
+
+    if (spawnExamination)
       createExaminedGame(game,plyr[0],plyr[1]);
     break;
+
   case -3: // isolated position (e.g.: sposition command)
     game=global.nextFreeGameId(10000);
     createScratchPos(game,plyr[0],plyr[1]);
