@@ -1,10 +1,10 @@
-/* $Id: proto_xboard.cc,v 1.15 2001/09/22 14:43:02 bergo Exp $ */
+/* $Id: proto_xboard.cc,v 1.18 2002/03/11 14:00:35 bergo Exp $ */
 
 /*
 
     eboard - chess client
     http://eboard.sourceforge.net
-    Copyright (C) 2000-2001 Felipe Paulo Guazzi Bergo
+    Copyright (C) 2000-2002 Felipe Paulo Guazzi Bergo
     bergo@seul.org
 
     This program is free software; you can redistribute it and/or modify
@@ -190,8 +190,9 @@ void XBoardProtocolParser::sendMove(int x1,int y1,int x2,int y2,int prom) {
   refgame=getGame();
   LegalityBackup=CurrentPosition;
 
-  CurrentPosition.stdNotationForMove(x1,y1,x2,y2,
-				     (prom?global.promotion->getPiece():EMPTY),mymove,refgame->Variant);
+  pp = prom ? global.promotion->getPiece() : EMPTY;
+
+  CurrentPosition.stdNotationForMove(x1,y1,x2,y2,pp,mymove,refgame->Variant);
   CurrentPosition.moveCartesian(x1,y1,x2,y2,Variant);
 
   if (EngineWhite)
@@ -261,7 +262,7 @@ void XBoardProtocolParser::gameOver(ExtPatternMatcher &pm,GameResult gr,
     }
   }
 
-  finalize();
+  global.network->close();
 }
 
 void XBoardProtocolParser::resign() {
